;*******************************************************************************
;   BASE.ASM -- DSP-BASED CCD CONTROLLER PROGRAM
;*******************************************************************************
    PAGE    110,60,1,1
    TABS    4
    NOLIST
;***** SYS=  (see makefile) ****************************************************
;  0=DEFAULT        10=SITE2K(1amp)     20=LDSS4 (4amp)
;  1=LAB            11=SITE2K(2amp)     21=LDSS2 (2amp)   
;  2=MIKE_BLUE      12=MITLL            22=CCD231 (4amp)
;  3=MIKE_RED       13=WF4K(1amp)       
;  4=SITe           14=CCD47
;  5=WING_CHIP                          30=IMACS (8amp)
;  6=BNC                                31=IMACSII (8amp)
;  7=MAGE           
;  8=WF4K (2amp)
;  9=PFS4K
;*******************************************************************************
;   DEFINITIONS & POINTERS
;   XMEM $800=2048  YMEM $800=2048  PMEM $1000=4096
;*******************************************************************************
START       EQU     $000100             ; program start location
SEQ         EQU     $000006             ; seq fragment length
DZ          EQU     $001000             ; DAC zero volt offset
XINIT       EQU     $0

;*******************************************************************************
;   BASE EXTERNAL PERIPHERAL DEFINITIONS
;*******************************************************************************
SEQREG      EQU     $FFFF88             ; external CCD clock register
TXREG       EQU     $FFFF89             ; external TX register
RXREG       EQU     $FFFF8A             ; external RX register
ADC         EQU     $FFFC00             ; A/D converters
ADC_A       EQU     $FFFC00             ; A/D converters
ADC_B       EQU     $FFFC01             ;
ADC_C       EQU     $FFFC20             ;
ADC_D       EQU     $FFFC21             ;
ADC_E       EQU     $FFFC40             ;
ADC_F       EQU     $FFFC41             ;
ADC_G       EQU     $FFFC60             ;
ADC_H       EQU     $FFFC61             ;
SIG_AB      EQU     $FFFC10             ; A+B bias
SIG_CD      EQU     $FFFC30             ; C+D
SIG_EF      EQU     $FFFC50             ; E+F
SIG_GH      EQU     $FFFC70             ; G+H
CLK_AB      EQU     $FFFD00             ; A+B clocks
CLK_CD      EQU     $FFFD40             ; C+D
CLK_EF      EQU     $FFFD80             ; E+F
CLK_GH      EQU     $FFFDC0             ; G+H
HR          EQU     $FFFF80             ; Heater resistors

;*******************************************************************************
;   DSP56300 INTERNAL PERIPHERAL DEFINITIONS
;*******************************************************************************
IPRC        EQU     $FFFFFF             ; Interrupt priority register (core)
IPRP        EQU     $FFFFFE             ; Interrupt priority register (periph)
PCTL        EQU     $FFFFFD             ; PLL control register
BCR         EQU     $FFFFFB             ; Bus control register (wait states)
AAR0        EQU     $FFFFF9             ; Address attribute register 0
AAR1        EQU     $FFFFF8             ; Address attribute register 1
AAR2        EQU     $FFFFF7             ; Address attribute register 2
AAR3        EQU     $FFFFF6             ; Address attribute register 3
IDR         EQU     $FFFFF5             ; ID Register
PDRB        EQU     $FFFFC9             ; Port B (HOST) GPIO data
PRRB        EQU     $FFFFC8             ; Port B (HOST) GPIO direction
PCRB        EQU     $FFFFC4             ; Port B (HOST) control register
PCRC        EQU     $FFFFBF             ; Port C (ESSI_0) control register
PRRC        EQU     $FFFFBE             ; Port C (ESSI_0) direction
PDRC        EQU     $FFFFBD             ; Port C (ESSI_0) data
TXD         EQU     $FFFFBC             ; ESSI0 Transmit Data Register 0
RXD         EQU     $FFFFB8             ; ESSI0 Receive Data Register
SSISR       EQU     $FFFFB7             ; ESSI0 Status Register
CRB         EQU     $FFFFB6             ; ESSI0 Control Register B
CRA         EQU     $FFFFB5             ; ESSI0 Control Register A
PCRD        EQU     $FFFFAF             ; Port D (ESSI_1) control register
PRRD        EQU     $FFFFAE             ; Port D (ESSI_1) direction
PDRD        EQU     $FFFFAD             ; Port D (ESSI_1) data
PCRE        EQU     $FFFF9F             ; Port E (SCI) control register
PRRE        EQU     $FFFF9E             ; Port E (SCI) data direction
PDRE        EQU     $FFFF9D             ; Port E (SCI) data
TCSR0       EQU     $FFFF8F             ; TIMER0 Control/Status Register
TLR0        EQU     $FFFF8E             ; TIMER0 Load Reg
TCPR0       EQU     $FFFF8D             ; TIMER0 Compare Register
TCR0        EQU     $FFFF8C             ; TIMER0 Count Register
TCSR1       EQU     $FFFF8B             ; TIMER1 Control/Status Register
TLR1        EQU     $FFFF8A             ; TIMER1 Load Reg
TCPR1       EQU     $FFFF89             ; TIMER1 Compare Register
TCR1        EQU     $FFFF88             ; TIMER1 Count Register
TCSR2       EQU     $FFFF87             ; TIMER2 Control/Status Register
TLR2        EQU     $FFFF86             ; TIMER2 Load Reg
TCPR2       EQU     $FFFF85             ; TIMER2 Compare Register
TCR2        EQU     $FFFF84             ; TIMER2 Count Register
TPLR        EQU     $FFFF83             ; TIMER Prescaler Load Register
TPCR        EQU     $FFFF82             ; TIMER Prescalar Count Register
DSR0        EQU     $FFFFEF             ; DMA source address
DDR0        EQU     $FFFFEE             ; DMA dest address
DCO0        EQU     $FFFFED             ; DMA counter
DCR0        EQU     $FFFFEC             ; DMA control register

;*******************************************************************************
;   BASE REGISTER DEFINITIONS & OTHER POINTERS
;*******************************************************************************
CMD         EQU     $000000             ; command word/flags from host
OPFLAGS     EQU     $000001             ; operational flags
NROWS       EQU     $000002             ; number of rows
NCOLS       EQU     $000003             ; number of columns
NFT         EQU     $000004             ; number of rows for frame transfer
NFLUSH      EQU     $000005             ; number of columns to flush
NCH         EQU     $000006             ; number of channels (detectors or amps)
NPIX        EQU     $000007             ; (not used)
VBIN        EQU     $000008             ; vertical (parallel) binning
HBIN        EQU     $000009             ; horizontal (serial) binning
VSKIP       EQU     $00000A             ; V prescan or offset (rows)
HSKIP       EQU     $00000B             ; H prescan or offset (columns)
VSUB        EQU     $00000C             ; V subraster size
HSUB        EQU     $00000D             ; H subraster size
NEXP        EQU     $00000E             ; number of exposures
NSHUFFLE    EQU     $00000F             ; number of rows for charge shuffle

EXP_TIME    EQU     $000010             ; exposure time (ms)
TEMP        EQU     $000011             ; temp sensor reading(s)
GAIN        EQU     $000012             ; sig_proc gain
USEC        EQU     $000013             ; sig_proc sample time
OPCH        EQU     $000014             ; sig_proc output channel
HDIR        EQU     $000015             ; serial clock direction
HCLKS       EQU     $000015             ; horizontal clock direction
LINK        EQU     $000016             ; optical link 0=wire 1=single_fiber
;WM          EQU     $000017             ; window mode (full/subraster)

ETIMER      EQU     $000018             ; exposure countdown timer
ESTATE      EQU     $000019             ; shutter open/closed
TEC         EQU     $00001A             ; TEC settting
PID         EQU     $00001B             ; temp PID control loop $0=OFF $1=ON
PID_TIME    EQU     $00001C             ; last PID adjustment time
NLOOP       EQU     $00001D             ; loop counter

TDI         EQU     $000020             ; TDI mode $0=off $1=on
TDI_TIME    EQU     $000021             ; TDI timer (cycles per tdi shift)
TDI_BUSY    EQU     $000022             ; TDI 0=ready 1=busy

SCLKS       EQU     $000030             ; serial clocks
SCLKS_FL    EQU     $000031             ; serial flush clocks
INT_X       EQU     $000032             ; reset and integrate clocks
NDMA        EQU     $000033             ; (not used)

HEAT        EQU     $000100             ; Heater settings
TCCD        EQU     $000108             ; CCD temp sensor reading(s)
TSET        EQU     $000110             ; CCD temp set point(s)
VBIAS       EQU     $000120             ; bias voltages
VCLK        EQU     $000180             ; clock voltages
PIDSP       EQU     $000200             ; PID scratch pad
PIX         EQU     $000300             ; start address for pixel data

E0          EQU     PIDSP+$0            ; PID temp control
E1          EQU     PIDSP+$8            ; scratch storage area
H0          EQU     PIDSP+$10           ;
H1          EQU     PIDSP+$18           ;

;*****************************************************************************
;   SEQUENCE FRAGMENT STARTING ADDRESSES (& OTHER POINTERS)
;*****************************************************************************
MPP         EQU     $000040             ; MPP/hold state
IPCLKS      EQU     $000042             ; input clamp
TCLKS       EQU     $000044             ; Temperature monitor clocks
PCLKS_FT    EQU     $000048             ; parallel frame transfer
PCLKS_RD    EQU     $000050             ; parallel read-out transfer
PCLKS_FL    EQU     $000058             ; parallel flush transfer
INT_L       EQU     $000060             ; reset and integrate left
INT_R       EQU     $000070             ; reset and integrate right
SCLKS_L     EQU     $000080             ; serial clocks shift left
SCLKS_R     EQU     $000088             ; serial clocks shift right
SCLKS_B     EQU     $000090             ; serial clocks shift both
SCLKS_FLL   EQU     $000098             ; serial clocks flush left
SCLKS_FLR   EQU     $0000A0             ; serial clocks flush right
SCLKS_FLB   EQU     $0000A8             ; serial clocks flush both
SCLKS_FF    EQU     $0000B0             ; serial clocks fast flush
PCLKS_FWD   EQU     $0000B8             ; parallel charge shuffle (forward)
PCLKS_REV   EQU     $0000C0             ; parallel charge shuffle (reverse)

;*******************************************************************************
;   MEMORY CONFIGURATION REGISTERS
;*******************************************************************************
;                                           Y X P
;                   BAC            BNC  7 6 5 4 3 2 10
;   AAR3 (PERIPH)   1111.1111.1111-1100-0-0-1-0-0-0-01  = $FFFC21
;   AAR2 (SRAM)     0000.0100.0000-0110-0-0-1-1-0-1-01  = $040635
;   AAR1 (FLASH)    1101.0000.0000-1011-0-0-0-0-1-0-01  = $D00B09
;   AAR0 (SRAM)     0000.0000.0000-0101-0-0-1-1-0-0-01  = $000531
;
;                   MISC  DEF AA3 AA2  AAR1  AAR0
;   BCR:            000-00111-001-111-11111-00001       = $073FE1
;   Waitstates:       0     7   1   7    31     1
;
;   BCR:            000-00111-011-111-11111-00001       = $077FE1
;   Waitstates:       0     7   3   7    31     1
;
;   BCR:            000-00111-101-111-11111-00001       = $07BFE1
;   Waitstates:       0     7   5   7    31     1

WS1         EQU     $073FE1             ; 1 PERIPH 1 SRAM 31 EPROM
WS3         EQU     $077FE1             ; 3 PERIPH 1 SRAM 31 EPROM
WS5         EQU     $07BFE1             ; 5 PERIPH 1 SRAM 31 EPROM
WS7         EQU     $07FFE1             ; 7 PERIPH 1 SRAM 31 EPROM
WS12        EQU     $072BE2             ; 1 PERIPH 2 SRAM 31 EPROM
WS32        EQU     $076BE2             ; 3 PERIPH 2 SRAM 31 EPROM
WS52        EQU     $07ABE2             ; 5 PERIPH 2 SRAM 31 EPROM

;*******************************************************************************
;   COMPILE-TIME OPTIONS
;*******************************************************************************
    IF (SYS==0)                         ; BASE SYSTEM / TEMPLATE
    include 'dev_base.asm'
    ENDIF

    IF (SYS==1)                         ; LAB SYSTEM
    include 'dev_lab.asm'
    ENDIF

    IF (SYS==2)                         ; MIKE_BLUE (E2V CCD44-82)
    include 'dev_mike_blue.asm'
    ENDIF

    IF (SYS==3)                         ; MIKE_RED (E2V CCD44-82)
    include 'dev_mike_red.asm'
    ENDIF

    IF (SYS==4)                         ; SITe ST-002A
    include 'dev_site.asm'
    ENDIF

    IF (SYS==5)                         ; WING_CHIP (STA 2560x512)
    include 'dev_wingchip.asm'
    ENDIF

    IF (SYS==6)                         ; B&C (E2V CCD42-10)
    include 'dev_bnc.asm'
    ENDIF

    IF (SYS==7)                         ; MAGE (E2V CCD42-20)
    include 'dev_mage.asm'
    ENDIF

    IF (SYS==8||SYS==13)                ; WF4K (STA0500A)
    include 'dev_wf4k.asm'
    ENDIF

    IF (SYS==9)                         ; PFS4K (STA0500A)
    include 'dev_pfs4k.asm'
    ENDIF

    IF (SYS==10||SYS==11)               ; SITE2K (SI-424A)
    include 'dev_site2k.asm'
    ENDIF

    IF (SYS==12)                        ; MIT/LL CCID-20
    include 'dev_mitll.asm'
    ENDIF

    IF (SYS==14)                        ; E2V CCD47-20
    include 'dev_ccd47.asm'
    ENDIF

    IF (SYS==20||SYS==21)               ; LDSS (STA0500A)
    include 'dev_ldss.asm'
    ENDIF

    IF (SYS==22||SYS==23)               ;  E2V CCD231
    include 'dev_ccd231.asm'
    ENDIF

    IF (SYS==30)                        ; IMACS I (SITe ST-002A)
    include 'dev_imacsI.asm'
    ENDIF

    IF (SYS==31)                        ; IMACS II (E2V CCD44-82)
    include 'dev_imacsII.asm'
    ENDIF

    IF (SYS==32)                        ; IMACS III (E2V CCD44-82)
    include 'dev_imacsIII.asm'
    ENDIF

;*******************************************************************************
;   INITIALIZE X MEMORY (PARAMETERS & PERIPHERALS)
;*******************************************************************************
;            ORG     X:CMD               ; CCD control information
;            DC      $000000             ; CMD/FLAGS
;            DC      $000000             ; OPFLAGS
;            DC      INIT_NROWS          ; NROWS
;            DC      INIT_NCOLS          ; NCOLS
;            DC      INIT_NFT            ; NFT
;            DC      INIT_NFLUSH         ; NFLUSH
;            DC      INIT_NCH            ; NCH
;            DC      $000001             ; NPIX (not used)
;            DC      INIT_VBIN           ; VBIN
;            DC      INIT_HBIN           ; HBIN
;            DC      INIT_VSKIP          ; VSKIP
;            DC      INIT_HSKIP          ; HSKIP
;            DC      $000000             ; VSUB
;            DC      $000000             ; HSUB
;            DC      $000001             ; NEXP (not used)
;            DC      $000001             ; NSHUFFLE
; 
;            ORG     X:EXP_TIME
;            DC      $0003E8             ; EXP_TIME
;            DC      $000000             ; TEMP
;            DC      INIT_GAIN           ; GAIN
;            DC      INIT_USEC           ; USEC
;            DC      INIT_OPCH           ; OPCH
;            DC      INIT_SCLKS          ; HCLKS
;
;            ORG     X:ETIMER
;            DC      $000000             ; ETIMER
;            DC      $000000             ; ESTATE
;            DC      $000000             ; 
;            DC      INIT_PID            ; PID FLAG $0=OFF $1=ON
;            DC      $000000             ; PID_TIME
;            DC      $000000             ; NLOOP
;            DC      $000000             ;
;            DC      $000000             ;

;            ORG     X:SCLKS
;            DC      $000000             ; SCLKS
;            DC      $000000             ; SCLKS_FL
;            DC      $000000             ; INT_X

            ORG     X:PIDSP             ;
            BSC     $8,$000000          ; BLOCK-FILL WITH ZEROs
            BSC     $8,$000000          ;
            BSC     $8,$000000          ;
            BSC     $8,$000000          ;

            ORG     X:HEAT
            BSC     $8,(DZ+0000)        ; BLOCK-FILL HEAT[0..7] (.05 mA/DN)

;    IF (SYS<20)                         ; non-IMACS non-LDSS
;            ORG     X:VBIAS+16          ; BLOCK-FILL BIAS VOLTAGES
;            BSC     48,(DZ+0000)        ; FOR CH C,D,E,F,G,H
;
;            ORG     X:VCLK+32           ; BLOCK-FILL CLOCK VOLTAGES
;            BSC     32,(DZ+0000)        ; FOR CH C,D
;    ENDIF

;    IF (SYS<30)                         ; non-IMACS
;            ORG     X:VCLK+64           ; BLOCK-FILL CLOCK VOLTAGES
;            BSC     64,(DZ+0000)        ; FOR CH E,F,G,H
;    ENDIF

;            INCLUDE 'codes.asm'

;*******************************************************************************
;   GENERAL COMMENTS
;*******************************************************************************
; Hardware RESET causes download from serial port (code in EPROM)
; R0 is a pointer to sequence fragments
; R1 is a pointer used by send/receive routines
; R2 is a pointer to the current data location
; See dspdvr.h for command and opflag definitions
;*******************************************************************************
;   OPFLAG DEFINITIONS (BY BIT #)
;*******************************************************************************
;   0 NO-SEND           4 AUTO-READ     8               C    
;   1 AUTO-FLUSH        5 AUTO-WIPE     9               D TDI-MODE
;   2 AUTO-SHUTTER      6 AUTO-SEND     A               E WINDOW-MODE(H2RG)
;   3                   7 RESET-READ    B               F EXP-IN-PROGRESS
;*******************************************************************************
;   INITIALIZE INTERRUPT VECTORS
;*******************************************************************************
            ORG     P:$0000
            JMP     START
;*******************************************************************************
;   MAIN PROGRAM
;*******************************************************************************
            ORG     P:START
SET_MODE    ORI     #$3,MR                  ; mask all interrupts
            ORI	    #$40,EOM                ; disble AA priority
            MOVEP   #$FFFC21,X:AAR3         ; PERIPH  $FFF000--$FFFFFF
            MOVEP   #$040635,X:AAR2         ; SRAM XY $040000--$07FFFF 256K A18
            MOVEP   #$D00B09,X:AAR1         ; PROM P  $D00000--$D01FFF 8K
            MOVEP   #$000531,X:AAR0         ; SRAM XY $000000--$00FFFF 512K
            MOVEP   #WS1,X:BCR              ; Set periph wait states
            MOVE    #SEQ-1,M0               ; Set sequencer address modulus

PLL_SETUP   MOVEP   #$078007,X:PCTL         ; set PLL parameters

SSI_SETUP   MOVEP   #>$032070,X:CRB         ; async, LSB, enable TE RE
            MOVEP   #>$140801,X:CRA         ; 20 Mbps, 16 bit word
            MOVEP   #>$3F,X:PCRC            ; enable ESSI

PORTB_SETUP MOVEP   #>$1,X:PCRB             ; set PB[15..0] GPIO
            MOVEP   #>$FF,X:PRRB            ; TM GAIN .. OPT3V

PORTD_SETUP MOVEP   #>$0,X:PCRD             ; set PD[0..5] GPIO
            MOVEP   #>$2F,X:PRRD            ; S0 S1 SFS SCK SRD STD
            MOVEP   #>$4,X:PDRD             ; set port D

PORTE_SETUP MOVEP   #$0,X:PCRE              ; disable SCI
            MOVEP   #>$6,X:PRRE             ; PE0=STAT PE1=SHUTTER PE2=LED
            MOVEP   #$2,X:PDRE              ; close shutter

            JSR     INIT_MEM                ; load initial values into memory

SET_TIMER   MOVEP   #>$9C3F,X:TPLR          ; prescaler ($9C3F=1ms @80MHz)
            MOVEP   #$300A10,X:TCSR0        ; Pulse mode, no prescale
            MOVEP   #$0,X:TLR0              ; timer reload value
            MOVEP   X:USEC,X:TCPR0          ; timer compare value
            MOVEP   #$308A10,X:TCSR1        ; Pulse mode, prescaled
            MOVEP   #$0,X:TLR1              ; timer reload value
            MOVEP   X:EXP_TIME,X:TCPR1      ; timer compare value
            MOVEP   #$308810,X:TCSR2        ; Pulse mode, prescaled, free-run
            MOVEP   #$0,X:TLR2              ; timer reload value
            MOVEP   #$0,X:TCPR2             ; timer compare value
            BSET    #$0,X:TCSR2             ; start timer

STATE_SETUP MOVE    #$0,X0                  ; set up exposure state
            MOVE    X0,X:ESTATE             ;
            MOVE    X0,X:ETIMER             ;

DMA_SETUP   MOVEP   #PIX,X:DSR0             ; set DMA source
            MOVEP   #$0,X:DCO0              ; set DMA counter
FIBER       JCLR    #$8,X:PDRB,RS485        ; set up optical
            MOVEP   #>TXREG,X:DDR0          ; set DMA destination
            MOVEP   #>$080255,X:DCR0        ; DMA word xfer, /IRQA, src+1
RS485       JSET    #$8,X:PDRB,ENDDP        ; set up RS485
            MOVEP   #>TXD,X:DDR0            ; DMA destination
            MOVEP   #>$085A51,X:DCR0        ; DMA word xfer, TDE0, src+1
ENDDP       NOP                             ;

INIT_SETUP  JSR     MPPHOLD                 ;
            JSR     SET_DACS                ;
            JSR     SET_HEAT                ;
            JSR     SET_GAIN                ;
            JSR     SET_SCLKS               ;
            JSR     SET_OPCH                ;
            JSR     TEMP_READ               ;
            BSET    #$9,X:CMD               ; set autowipe on

WAIT_CMD    JCLR    #$8,X:PDRB,WAITB        ; check for cmd ready
            JCLR    #$A,X:PDRB,ECHO         ; fiber link (single-fiber)
WAITB       JSET    #$8,X:PDRB,ENDW         ;
            JCLR    #7,X:SSISR,ECHO         ; wire link
ENDW        NOP                             ;
            JSR     READ16                  ;
            MOVE    A1,X:CMD                ; store command word
            JSR     CMD_FIX                 ; interpret command + ack

ECHO        JCLR    #$1,X:CMD,GET           ; test for DSP_ECHO command
            JSR     READ16                  ;
            JSR     WRITE16                 ;
            BCLR    #$1,X:CMD               ;

GET         JCLR    #$2,X:CMD,PUT           ; test for DSP_GET command
            JSR     MEM_SEND                ;
            BCLR    #$2,X:CMD               ;

PUT         JCLR    #$3,X:CMD,EXP_START     ; test for DSP_PUT command
            JSR     MEM_LOAD                ;
            BCLR    #$3,X:CMD               ;

EXP_START   JCLR    #$6,X:CMD,EXP_ABORT     ; test for EXPOSE command
            JSR     MPPHOLD                 ;
            MOVE    #PIX,R2                 ; set data pointer
            MOVE    X:NCH,A                 ; set DMA counter & 
            SUB     #>$1,A                  ; output amp
            MOVE    N5,R5                   ; set output amp
            MOVE    A1,X:DCO0               ;
            MOVEP   X:EXP_TIME,X:TCPR1      ; timer compare value
            MOVEP   #$0,X:TLR1              ; timer reload value
            BSET    #$F,X:CMD               ; perpare to start timer
            BSET    #$F,X:OPFLAGS           ; set exp_in_progress flag
            BCLR    #$6,X:CMD               ;
            MOVE    #$0,X0                  ; zero TDI counter for IMACS
            MOVE    X0,X:NLOOP              ;

            JCLR    #$1,X:OPFLAGS,EXP_ABORT ; check for AUTO_FLUSH
            BSET    #$4,X:CMD               ;

EXP_ABORT   JCLR    #$0,X:CMD,FASTFLUSH     ; check for abort flag
            BCLR    #$0,X:TCSR1             ; stop timer, clear flag
            BCLR    #$F,X:OPFLAGS           ; clear exp_in_progress flag
            BSET    #$7,X:CMD               ; prep to close shutter

FASTFLUSH   JCLR    #$4,X:CMD,EXP_PAUSE     ; test for FLUSH command
            JSR     FLUSHFRAME              ; fast FLUSH
            JSR     FLUSHLINE               ; clear serial register
            BCLR    #$4,X:CMD               ;

EXP_PAUSE   JCLR    #$E,X:CMD,EXP_RESUME    ; check for PAUSE command
            BCLR    #$0,X:TCSR1             ; stop timer, clear flag
            MOVEP   X:TCR1,X:TLR1           ; update reload value
            BCLR    #$E,X:CMD               ;

            JCLR    #$2,X:OPFLAGS,EXP_RESUME ; check for AUTO_SHUTTER
            BSET    #$7,X:CMD               ; prepare to close shutter

EXP_RESUME  JCLR    #$F,X:CMD,BEAM_ON       ; check for RESUME command
            BSET    #$0,X:TCSR1             ; start timer
            BCLR    #$F,X:CMD               ;

ER1         JCLR    #$2,X:OPFLAGS,BEAM_ON   ; check for AUTO_SHUTTER
            BSET    #$5,X:CMD               ; prepare to open shutter

BEAM_ON     JCLR    #$5,X:CMD,EXP_STATUS    ; test for open shutter
            BCLR    #$1,X:PDRE              ; open shutter
    IF (SYS<30)                             ; non-IMACS
            BSET    #$0,X:ESTATE            ; update shutter status bit
    ENDIF                                   ;
            BCLR    #$5,X:CMD               ;


EXP_STATUS  JCLR    #$F,X:OPFLAGS,BEAM_OFF  ; check exp_in_progress flag
            MOVE    X:EXP_TIME,A            ;
            MOVEP   X:TCPR1,X0              ;
            CMP     X0,A                    ;
            JEQ     CK_TDI                  ;
            MOVE    A1,X:TCPR1              ;

CK_TDI      JCLR    #$D,X:OPFLAGS,CK_STAT   ; check for TDI mode (for IMACS)
            MOVE    X:NEXP,A                ; check exposure loop counter
            MOVE    X:NLOOP,X0              ;
            CMP     X0,A                    ; (NEXP - NLOOP)
            JGT     BEAM_OFF                ;
            BCLR    #$13,X:CMD              ; stop readlines
            JMP     CKS1                    ;

CK_STAT     MOVE    X:EXP_TIME,A            ; compute time left,
            MOVEP   X:TCR1,X0               ; busy-wait when only 20ms
            SUB     X0,A                    ;
            MOVE    #>$14,X0                ; X0=20ms
            MOVE    A1,X:ETIMER             ; A1=time left
            MOVE    #$0,A2                  ; note: clear sign extension
            CMP     X0,A                    ; check A-X0
            JGE     BEAM_OFF                ;

CKS1        JCLR    #$15,X:TCSR1,*          ; wait for TCF flag
            BCLR    #$0,X:TCSR1             ; stop timer, clear flag
            MOVE    #$0,X0                  ;
            MOVE    X0,X:ETIMER             ;
            BCLR    #$F,X:OPFLAGS           ; clear exp_in_progress flag

;;          MOVE    X:NCH,A                 ; set DMA counter
;;          SUB     #>$1,A                  ;
;;          NOP                             ;
;;          MOVE    A1,X:DCO0               ;

OPT_A       JCLR    #$2,X:OPFLAGS,OPT_B     ; check for AUTO_SHUTTER
            BSET    #$7,X:CMD               ; prep to close shutter
OPT_B       JCLR    #$4,X:OPFLAGS,BEAM_OFF  ; check for AUTO_READ
            BSET    #$8,X:CMD               ; prep for full readout

BEAM_OFF    JCLR    #$7,X:CMD,SHUFFLE_F     ; test for shutter close
            BSET    #$1,X:PDRE              ; close shutter
    IF (SYS<30)                             ; non-IMACS
            BCLR    #$0,X:ESTATE            ; update shutter status bit
    ENDIF                                   ;
            BCLR    #$7,X:CMD               ;

SHUFFLE_F   JCLR    #$10,X:CMD,SHUFFLE_R    ;
            JSR     FSHUFFLE                ;
            BCLR    #$10,X:CMD              ;

SHUFFLE_R   JCLR    #$11,X:CMD,FLUSH_LINE   ;
            JSR     RSHUFFLE                ;
            BCLR    #$11,X:CMD              ;

FLUSH_LINE  JCLR    #$14,X:CMD,READ_LINE    ;
            JSR     FLUSHROWS               ;
            JSR     FLUSHLINE               ;
            BCLR    #$14,X:CMD              ;

READ_LINE   JCLR    #$13,X:CMD,READ_CCD     ;
            MOVE    X:NCH,A                 ; set DMA counter
            SUB     #>$1,A                  ;
            NOP                             ;
            MOVE    A1,X:DCO0               ;
            JSR     PARALLEL                ;
            BSET    #$0,X:OPFLAGS           ; set first pixel flag
            JSR     PRESKIP                 ;
            JSR     READPIX                 ;
            BCLR    #$0,X:OPFLAGS           ; clear no-send flag
            JSR     READLINE                ;

            JSET    #$D,X:OPFLAGS,RL0       ; TDI mode for IMACS
            BCLR    #$13,X:CMD              ;

RL0         JCLR    #$D,X:OPFLAGS,READ_CCD  ; TDI mode for IMACS
            JCLR    #$15,X:TCSR1,*          ; wait for usec TCF flag
            BSET    #$15,X:TCSR1            ; clear flag
RL1         MOVE    X:NLOOP,A               ; increment loop counter
            ADD     #>$1,A                  ;
            NOP                             ;
            MOVE    A1,X:NLOOP              ;

READ_CCD    JCLR    #$8,X:CMD,AUTO_WIPE     ; test for READCCD command
            MOVE    X:NCH,A                 ; set DMA counter
            SUB     #>$1,A                  ;
            NOP                             ;
            MOVE    A1,X:DCO0               ;
;           JSR     FRAME                   ; frame transfer ccd47
            JSR     FLUSHLINE               ; flush serial register
            BSET    #$0,X:OPFLAGS           ; set no-send flag
            DUP     $10                     ;
            JSR     READLINE                ;
            ENDM                            ;
            DO      X:NROWS,END_READ        ; read the array
            JSR     PARALLEL                ;
            BSET    #$0,X:OPFLAGS           ; set no-send flag
            JSR     PRESKIP                 ;
            JSR     READPIX                 ;
            BCLR    #$0,X:OPFLAGS           ; clear no-send flag
            JSR     READLINE                ;
            NOP                             ;
END_READ    NOP                             ;
            JSR     MPPHOLD                 ;
            BCLR    #$8,X:CMD               ;

AUTO_WIPE   JCLR    #$9,X:CMD,HH_DACS       ; test for AUTOWIPE command
            JSET    #$F,X:OPFLAGS,HH_DACS   ; check exp_in_progress flag
            BSET    #$5,X:OPFLAGS           ;
            BSET    #$0,X:OPFLAGS           ; set no-send flag
            MOVE    X:VBIN,N0               ;
            MOVE    X:HBIN,N1               ;
            MOVE    #>$4,Y0                 ;
            MOVE    Y0,X:VBIN               ;
            MOVE    Y0,X:HBIN               ;
            JSR     PARALLEL                ; flush one parallel row
            JSR     READLINE                ;
            JSR     MPPHOLD                 ;
            MOVE    N0,X:VBIN               ;
            MOVE    N1,X:HBIN               ;
            BCLR    #$9,X:CMD               ;

HH_DACS     JCLR    #$A,X:CMD,HH_TEMP       ; test for HH_DACS command
            JSR     SET_DACS                ;
            BCLR    #$A,X:CMD               ;

HH_TEMP     JCLR    #$B,X:CMD,HH_TEC        ; test for HH_TEMP command
            JSR     TEMP_READ               ; perform housekeeping chores
            BCLR    #$B,X:CMD               ;

HH_TEC      JCLR    #$C,X:CMD,HH_OTHER      ; test for HH_TEC command
            JSR     SET_HEAT                ; set the heater currents
            BCLR    #$C,X:CMD               ;

HH_OTHER    JCLR    #$D,X:CMD,HH_PID        ; test for HH_OTHER command
            JSR     SET_GAIN                ;
            JSR     SET_SCLKS               ;
            JSR     SET_OPCH                ;
            JSR     SET_USEC                ;
            BCLR    #$D,X:CMD               ;

HH_PID      JCLR    #$0,X:PID,OPT_WIPE      ;
            MOVE    X:PID_TIME,X0           ;
            MOVE    X:TCR2,A                ;
            CMP     X0,A                    ;
            JLT     OPT_WIPE                ;
            ADD     #>$2710,A               ; $2710 = 10,000
            NOP                             ;
            MOVE    A1,X:PID_TIME           ;
            JSR     TEMP_READ               ;
            JSR     TEMP_PID                ;

OPT_WIPE    JCLR    #$5,X:OPFLAGS,END_CODE  ; check for AUTO_WIPE
            BSET    #$9,X:CMD

END_CODE    MOVE    X:PDRE,A                ; update shutter status bit
    IF (SYS==30||SYS==31||SYS==32)          ; IMACS
            AND     #$1,A                   ; (from JackRabbit controller)
            NOP                             ;
            MOVE    A1,X:ESTATE             ;
    ENDIF

    IF (SYS==9)                             ; PFS-only
EC0         JSET    #$0,X:PDRE,EC1          ; check TDI signal from PLC
            JSET    #$0,X:TDI_BUSY,EC1      ; 0=TDI-ready  1=TDI-busy
            BSET    #$11,X:CMD              ; set SHUFFLE_R cmd
            BSET    #$0,X:TDI_BUSY          ; set TDI-busy
            BSET    #$2,X:PDRE              ; LED on
EC1         JCLR    #$0,X:PDRE,EC2          ; check TDI signal from PLC
            BCLR    #$0,X:TDI_BUSY          ; clear TDI-busy
            BCLR    #$2,X:PDRE              ; LED off
EC2         NOP
    ENDIF

            JMP     WAIT_CMD                ; Get next command
            JMP     *                       ;

;*******************************************************************************
;   HOLD (MPP MODE)
;*******************************************************************************
MPPHOLD     MOVEP   X:MPP,Y:<<SEQREG        ;
            RTS                             ;

;*******************************************************************************
;   INPUT CLAMP
;*******************************************************************************
IPC_CLAMP   MOVEP   X:IPCLKS,Y:<<SEQREG     ;
            MOVE    #>$FA,X0                ; $FA0=50us
            REP     X0                      ;
            NOP                             ;
            MOVEP   X:(IPCLKS+1),Y:<<SEQREG ;
            REP     X0                      ;
            NOP                             ;
            RTS

;*******************************************************************************
;   FLUSHLINE  (FAST FLUSH)
;*******************************************************************************
;FLUSHLINE  MOVE    #SCLKS_FF,R0            ; initialize pointer
;           DO      #SEQ,ENDFF              ;
;           MOVEP   X:(R0)+,Y:<<SEQREG      ;
;           REP     #HOLD_FF                ;
;           NOP                             ;
;ENDFF      RTS                             ;

;*******************************************************************************
;   FLUSHLINE
;*******************************************************************************
FLUSHLINE   DO      X:NFLUSH,ENDFL          ;
FLPIX       MOVE    X:SCLKS_FL,R0           ; initialize pointer
            DO      #SEQ,ENDFLPIX           ;
            MOVEP   X:(R0)+,Y:<<SEQREG      ;
            REP     #HOLD_S                 ;
            NOP                             ;
ENDFLPIX    NOP                             ;
ENDFL       RTS

;*******************************************************************************
;   FLUSHPIX (HSKIP)
;*******************************************************************************
FLUSHPIX    DO      X:HSKIP,ENDFP           ;
            MOVE    X:SCLKS_FL,R0           ; initialize pointer
            DO      #SEQ,ENDHCLK            ;
            MOVEP   X:(R0)+,Y:<<SEQREG      ;
            REP     #HOLD_S                 ;
            NOP                             ;
ENDHCLK     NOP                             ;
ENDFP       RTS                             ;

;*******************************************************************************
;   FLUSHROWS (VSKIP)
;*******************************************************************************
FLUSHROWS   DO      X:VSKIP,ENDVSKIP        ;
            MOVE    #PCLKS_FL,R0            ; initialize pointer
            DO      #SEQ,ENDVCLK            ;
            MOVEP   X:(R0)+,Y:<<SEQREG      ;
            MOVE    #>HOLD_FL,X0            ;
            REP     X0                      ;
            NOP                             ;
ENDVCLK     NOP                             ;
ENDVSKIP    RTS                             ;

;*******************************************************************************
;   FLUSHFRAME
;*******************************************************************************
FLUSHFRAME  DO      X:NFT,ENDFLFR           ;
FL_CLOCKS   MOVE    #PCLKS_FL,R0            ; initialize pointer
            DO      #SEQ,ENDFLCLK           ;
            MOVEP   X:(R0)+,Y:<<SEQREG      ;
            MOVE    #>HOLD_FL,X0            ;
            REP     X0                      ;
            NOP                             ;
ENDFLCLK    NOP                             ;
ENDFLFR     RTS                             ;

;*****************************************************************************
;   PARALLEL TRANSFER (FRAME TRANSFER)
;*****************************************************************************
FRAME       MOVEP   X:(PCLKS_FT),Y:<<SEQREG ; 100 us CCD47 pause
            MOVE    #>$1F40,X0              ;
            REP     X0                      ; $1F40=100 usec
            NOP                             ;
            DO      X:NFT,ENDFT             ;
FT_CLOCKS   MOVE    #PCLKS_FT,R0            ; initialize seq pointer
            DO      #SEQ,ENDFTCLK           ;
            MOVEP   X:(R0)+,Y:<<SEQREG      ;
            MOVE    #>HOLD_FT,X0            ;
            REP     X0                      ;
            NOP                             ;
ENDFTCLK    NOP                             ;
ENDFT       RTS                             ;


;*******************************************************************************
;   PARALLEL TRANSFER (READOUT)
;*******************************************************************************
PARALLEL    DO      X:VBIN,ENDPT            ;
P_CLOCKS    MOVE    #PCLKS_RD,R0            ; initialize pointer
            DO      #SEQ,ENDPCLK            ;
            MOVEP   X:(R0)+,Y:<<SEQREG      ;
            MOVE    #>HOLD_P,X0             ;
            REP     X0                      ;
            NOP                             ;
ENDPCLK     NOP                             ;
ENDPT       NOP                             ;
            MOVE    #>HOLD_PL,X0            ; post-parallel, pre-line
            REP     X0                      ; settling time
            NOP                             ;
            RTS                             ;

;*******************************************************************************
;   PARALLEL TRANSFER (CHARGE SHUFFLE)
;*******************************************************************************
FSHUFFLE    MOVE    #PCLKS_FWD,R0           ; initialize pointer
            JMP     PSHUFFLE                ;
RSHUFFLE    MOVE    #PCLKS_REV,R0           ; initialize pointer
            NOP                             ;
PSHUFFLE    DO      X:NSHUFFLE,ENDPSH       ;
            DO      #SEQ,ENDPSHCLK          ;
            MOVEP   X:(R0)+,Y:<<SEQREG      ;
            MOVE    #>HOLD_P,X0             ;
            REP     X0                      ; $317=10us per phase
            NOP                             ;
ENDPSHCLK   NOP                             ;
ENDPSH      RTS                             ;

;*******************************************************************************
;   PRESKIP (HORIZONTAL)
;*******************************************************************************
PRESKIP     MOVE    X:HBIN,N1               ;
            MOVE    #>$1,Y0                 ;
            MOVE    Y0,X:HBIN               ;
            DO      X:HSKIP,ENDPSKIP        ;
            JSR     READPIX                 ;
            NOP                             ;
ENDPSKIP    MOVE    N1,X:HBIN               ;
            RTS                             ;

;*******************************************************************************
;   READLINE SUBROUTINE
;*******************************************************************************
READLINE    DO      X:NCOLS,ENDRL           ;
READPIX     MOVE    X:INT_X,R1              ;
            MOVE    X:SCLKS,R0              ; prep for serials
            MOVE    #PIX,R2                 ; prep for data storage
            MOVEP   X:(R1)+,Y:<<SEQREG      ; FRST=ON RG=ON
            DUP     HOLD_RG                 ; macro
            NOP                             ;
            ENDM                            ; end macro
            MOVEP   X:(R1)+,Y:<<SEQREG      ; RG=OFF
SERIAL      DO      X:HBIN,ENDSCLK          ;
S_CLOCKS    DUP     SEQ                     ; macro
            MOVEP   X:(R0)+,Y:<<SEQREG      ;
            DUP     HOLD_S                  ; macro
            NOP                             ;
            ENDM                            ; end macro
            ENDM                            ; end macro
ENDSCLK                                     ;
            REP     #HOLD_IPC               ; IPC ON
            NOP                             ; extra clamping time
            MOVEP   X:(R1)+,Y:<<SEQREG      ; FRST=OFF, preamp settling
            REP     #HOLD_SIG               ; settling time
            NOP                             ;
INT1        MOVEP   X:(R1)+,Y:<<SEQREG      ; FINT+=ON
SLEEP1      MOVE    X:USEC,X0               ; sleep USEC * 12.5ns
            REP     X0                      ;
            NOP                             ;
            MOVEP   #WS5,X:BCR              ;
            NOP                             ;
    IF (RLMODE==1)
            MOVE    Y:ADC_A,A               ;
            MOVE    A1,Y:(R2)+              ;
    ENDIF
    IF (RLMODE==2)
            MOVE    Y:ADC_A,A               ;
            MOVE    A1,Y:(R2)+              ;
            MOVE    Y:ADC_B,A               ;
            MOVE    A1,Y:(R2)+              ;
    ENDIF
    IF (RLMODE==3)
            MOVE    Y:ADC_A,A               ;
            MOVE    A1,Y:(R2)+              ;
            MOVE    Y:ADC_B,A               ;
            MOVE    A1,Y:(R2)+              ;
            MOVE    Y:ADC_C,A               ;
            MOVE    A1,Y:(R2)+              ;
            MOVE    Y:ADC_D,A               ;
            MOVE    A1,Y:(R2)+              ;
    ENDIF
            MOVEP   #WS1,X:BCR              ;
            NOP                             ;
            MOVEP   X:(R1)+,Y:<<SEQREG      ; FINT+=OFF
            MOVEP   X:(R1)+,Y:<<SEQREG      ; SW=OFF
            REP     #HOLD_SIG               ; settling time
            NOP                             ;
INT2        MOVEP   X:(R1)+,Y:<<SEQREG      ; FINT-=ON
SLEEP2      MOVE    X:USEC,X0               ; sleep USEC * 12.5ns
            REP     X0                      ;
            NOP                             ;
            MOVEP   #WS5,X:BCR              ;
            NOP                             ;
    IF (RLMODE==1)
            MOVE    Y:ADC_B,A               ;
            MOVE    A1,Y:(R2)+              ;
    ENDIF
    IF (RLMODE==2)
            MOVE    Y:ADC_C,A               ;
            MOVE    A1,Y:(R2)+              ;
            MOVE    Y:ADC_D,A               ;
            MOVE    A1,Y:(R2)+              ;
    ENDIF
    IF (RLMODE==3)
            MOVE    Y:ADC_E,A               ;
            MOVE    A1,Y:(R2)+              ;
            MOVE    Y:ADC_F,A               ;
            MOVE    A1,Y:(R2)+              ;
            MOVE    Y:ADC_G,A               ;
            MOVE    A1,Y:(R2)+              ;
            MOVE    Y:ADC_H,A               ;
            MOVE    A1,Y:(R2)+              ;
    ENDIF
            MOVEP   #WS1,X:BCR              ;
            NOP                             ;
            MOVEP   X:(R1)+,Y:<<SEQREG      ; FINT-=OFF
            REP     #HOLD_ADC               ;
            NOP                             ; settling time
            MOVEP   N5,X:DSR0               ; reset source address
CONVST      MOVEP   X:(R1)+,Y:<<SEQREG      ; /CONVST=ON
            NOP                             ;
            NOP                             ;
            MOVEP   X:(R1)+,Y:<<SEQREG      ; /CONVST=OFF MIN 100 NS
            MOVEP   X:(R1)+,Y:<<SEQREG      ; FRST=ON
            NOP                             ; settling time
            JSET    #$0,X:OPFLAGS,ENDCHK    ; check no-send flag
            BSET    #$17,X:DCR0             ; enable DMA
ENDCHK      NOP                             ;
ENDRL       NOP                             ;
            RTS                             ;

;*****************************************************************************
;   READLINE SUBROUTINE (FOR CCD47-20)
;*****************************************************************************
    IF (SYS==99)
READLINE    DO      X:NCOLS,ENDRL           ;
READPIX     MOVEP   X:(INT_L),Y:<<SEQREG    ; FRST=ON RG=ON
            DUP     HOLD_RG                 ; macro
            NOP                             ;
            ENDM                            ; end macro
            MOVEP   X:(INT_L+1),Y:<<SEQREG  ; RG=OFF            IPC ON
;            REP     #HOLD_IPC               ; settling time
;            NOP                             ;
            MOVEP   X:(INT_L+2),Y:<<SEQREG  ; FRST=OFF          IPC OFF
            REP     #HOLD_SIG               ; preamp settling time
            NOP                             ;
INT1        MOVEP   X:(INT_L+3),Y:<<SEQREG  ; FINT+=ON
SLEEP1      MOVE    X:USEC,X0               ; sleep USEC * 12.5ns
            REP     X0                      ;
            NOP                             ;
            MOVEP   X:(INT_L+4),Y:<<SEQREG  ; FINT+=OFF
SERIAL      MOVE    X:SCLKS,R0              ; serial transfer
            DO      X:HBIN,ENDSCLK          ;
S_CLOCKS    DUP     SEQ                     ;    macro
            MOVEP   X:(R0)+,Y:<<SEQREG      ;
            DUP     HOLD_S                  ;    macro
            NOP                             ;
            ENDM                            ;
            ENDM                            ;
ENDSCLK     REP     #HOLD_SIG               ; preamp settling time
            NOP                             ; (adjust with o'scope)
GET_DATA    MOVEP   #WS5,X:BCR              ;
            NOP                             ;
            NOP                             ;
            MOVEP   Y:<<ADC_A,A             ; read ADC
            MOVEP   Y:<<ADC_B,B             ; read ADC
            MOVEP   #WS1,X:BCR              ;
            NOP                             ;
INT2        MOVEP   X:(INT_H),Y:<<SEQREG    ; FINT-=ON
SLEEP2      MOVE    X:USEC,X0               ; sleep USEC * 20ns
            REP     X0                      ;
            NOP                             ;
            MOVEP   X:(INT_H+1),Y:<<SEQREG  ; FINT-=OFF
    IF (SYS==1)
DATA_MASK   AND     #>$3FFF,A               ; keep only 14-bits
            AND     #>$3FFF,B               ;
            BCHG    #$D,A1                  ; 2complement to binary
            BCHG    #$D,B1                  ;
    ENDIF
            MOVE    A1,Y:(PIX)              ;
            MOVE    B1,Y:(PIX+1)            ;
            REP     #HOLD_ADC               ; settling time
            NOP                             ; (adjust for best noise)
CONVST      MOVEP   X:(INT_H+2),Y:<<SEQREG  ; /CONVST=ON
            MOVEP   N5,X:DSR0               ; set DMA source
            NOP                             ;
            NOP                             ;
            MOVEP   X:(INT_H+3),Y:<<SEQREG  ; /CONVST=OFF MIN 40 NS
            MOVEP   X:(INT_H+4),Y:<<SEQREG  ; FRST=ON
            JSET    #$0,X:OPFLAGS,ENDCHK    ; check for first pixel
            BSET    #$17,X:DCR0             ; enable DMA
ENDCHK      NOP                             ;
ENDRL       RTS                             ;

    ENDIF

;*******************************************************************************
;   LOAD NEW DATA
;*******************************************************************************
MEM_LOAD    JSR     READ24                  ; get memspace/address
            MOVE    A1,R1                   ; load address into R1
            MOVE    A1,X0                   ; store memspace code
            JSR     READ24                  ; get data
            BCLR    #$17,R1                 ; clear memspace bit
X_LOAD      JSET    #$17,X0,Y_LOAD          ;
            MOVE    A1,X:(R1)               ; load x memory
Y_LOAD      JCLR    #$17,X0,END_LOAD        ;
            MOVE    A1,Y:(R1)               ; load y memory
END_LOAD    RTS                             ;

;*******************************************************************************
;   SEND MEMORY CONTENTS
;*******************************************************************************
MEM_SEND    JSR     READ24                  ; get memspace/address
            MOVE    A1,R1                   ; load address into R1
            MOVE    A1,X0                   ; save memspace code
            BCLR    #$17,R1                 ; clear memspace bit
X_SEND      JSET    #$17,X0,Y_SEND          ;
            MOVE    X:(R1),A1               ; send x memory
Y_SEND      JCLR    #$17,X0,SEND24          ;
            MOVE    Y:(R1),A1               ; send y memory
SEND24      JSR     WRITE24                 ;
            RTS                             ;

;*******************************************************************************
;   READ AND WRITE 16-BIT AND 24-BIT DATA (S4 VERSION)
;*******************************************************************************
READ16      JCLR    #$8,X:PDRB,RD16B        ; check RS485 or fiber
            JCLR    #$A,X:PDRB,*            ; wait for data in RXREG
            MOVE    Y:RXREG,A               ; bits 15..0
            AND     #>$FFFF,A               ;
RD16B       JSET    #$8,X:PDRB,ENDRD16      ; check RS485 or fiber
            JCLR    #7,X:SSISR,*            ; wait for RDRF to go high
            MOVE    X:RXD,A1                ; read from ESSI
            NOP                             ;
ENDRD16     RTS                             ; 16-bit word in A1 (was RTS)

WRITE16     JCLR    #$8,X:PDRB,WR16B        ; check RS485 or fiber
            MOVE    A1,Y:TXREG              ; write bits 15..0
WR16B       JSET    #$8,X:PDRB,ENDWR16      ;
            JCLR    #6,X:SSISR,*            ; wait for TDE
            MOVE    A1,X:TXD                ;
ENDWR16     RTS                             ;

READ24      JCLR    #$8,X:PDRB,RD24B        ; check RS485 or fiber
            JCLR    #$A,X:PDRB,*            ; wait for data in RXREG
            MOVE    Y:RXREG,A               ; bits 15..0
            AND     #>$FFFF,A               ;
            ASR     #$10,A,A                ; shift right 16 bits
            JCLR    #$A,X:PDRB,*            ; wait for data in RXREG
            MOVE    Y:RXREG,A1              ; bits 15..0
            ASL     #$10,A,A                ; shift left 16 bits
RD24B       JSET    #$8,X:PDRB,ENDRD24      ;
            JCLR    #7,X:SSISR,*            ; wait for RDRF to go high
            MOVE    X:RXD,A                 ; read from ESSI
            ASR     #$10,A,A                ; shift right 16 bits
            JCLR    #7,X:SSISR,*            ; wait for RDRF to go high
            MOVE    X:RXD,A1                ;
            ASL     #$10,A,A                ; shift left 16 bits
ENDRD24     RTS                             ; 24-bit word in A1

WRITE24     JCLR    #$8,X:PDRB,WR24B        ; check RS485 or fiber
            MOVE    A1,Y:TXREG              ; send bits 15..0
            ASR     #$10,A,A                ; right shift 16 bits
            REP     #$10                    ; wait for data sent
            NOP                             ;
            MOVE    A1,Y:TXREG              ; send bits 23..16
WR24B       JSET    #$8,X:PDRB,ENDWR24      ;
            JCLR    #6,X:SSISR,*            ; wait for TDE
            MOVE    A1,X:TXD                ; send bits 15..0
            ASR     #$10,A,A                ; right shift 16 bits
            NOP                             ; wait for flag update
            JCLR    #6,X:SSISR,*            ; wait for TDE
            MOVE    A1,X:TXD                ; send bits 23..16
ENDWR24     RTS                             ;

;*******************************************************************************
;   SET DAC VOLTAGES  DEFAULTS:  OD=24V  RD=12V  OG=ABG=-4V
;   PCLKS=+3V -9V SCLKS=+5V -5V RG=+12V 0V
;*******************************************************************************
SET_DACS    JSR     SET_VBIAS               ;
            JSR     SET_VCLKS               ;
            RTS                             ;

SET_VBIAS   MOVEP   #WS5,X:BCR              ; add wait states
            MOVE    #VBIAS,R3               ; voltage settings
            MOVE    #SIG_AB,R4              ; DAC address
            DO      #$10,ENDB0              ; set bias voltages
            MOVE    X:(R3)+,X0              ;
            MOVE    X0,Y:(R4)+              ;
ENDB0       NOP                             ;
    IF (SYS>=20)                            ;    IMACS/LDSS
            MOVE    #SIG_CD,R4              ; DAC address
            DO      #$10,ENDB1              ; set bias voltages
            MOVE    X:(R3)+,X0              ;
            MOVE    X0,Y:(R4)+              ;
ENDB1       NOP                             ;
    ENDIF
    IF (SYS>=30)                            ;    IMACS
            MOVE    #SIG_EF,R4              ; DAC address
            DO      #$10,ENDB2              ; set bias voltages
            MOVE    X:(R3)+,X0              ;
            MOVE    X0,Y:(R4)+              ;
ENDB2       NOP                             ;
            MOVE    #SIG_GH,R4              ; DAC address
            DO      #$10,ENDB3              ; set bias voltages
            MOVE    X:(R3)+,X0              ;
            MOVE    X0,Y:(R4)+              ;
ENDB3       NOP                             ;
    ENDIF
            MOVEP   #WS1,X:BCR              ;
            RTS                             ;

SET_VCLKS   MOVEP   #WS5,X:BCR              ; add wait states
SET_V0      MOVE    #VCLK,R3                ; voltage settings
            MOVE    #CLK_AB,R4              ; DAC address
            DO      #$20,ENDV0              ; set clock voltages
            MOVE    X:(R3)+,X0              ;
            MOVE    X0,Y:(R4)+              ;
ENDV0       NOP                             ;
    IF (SYS>=30)                            ; IMACS
            MOVE    #CLK_CD,R4              ; DAC address
            DO      #$20,ENDV1              ; set clock voltages
            MOVE    X:(R3)+,X0              ;
            MOVE    X0,Y:(R4)+              ;
ENDV1       NOP                             ;
            MOVE    #CLK_EF,R4              ; DAC address
            DO      #$20,ENDV2              ; set clock voltages
            MOVE    X:(R3)+,X0              ;
            MOVE    X0,Y:(R4)+              ;
ENDV2       NOP                             ;
SETV3       MOVE    #CLK_GH,R4              ; DAC address
            DO      #$20,ENDV3              ; set clock voltages
            MOVE    X:(R3)+,X0              ;
            MOVE    X0,Y:(R4)+              ;
ENDV3       NOP                             ;
    ENDIF
            MOVEP   #WS1,X:BCR              ; re-set wait states
            RTS                             ;

SET_HEAT    MOVEP   #WS5,X:BCR              ; add wait states
            MOVE    #HEAT,R3                ; heater settings
            MOVE    #HR,R4                  ; DAC address

            DO      #$8,END_HR              ; set heat voltages
            MOVE    X:(R3)+,X0              ;
            MOVE    X0,Y:(R4)+              ;
END_HR      MOVEP   #WS1,X:BCR              ; re-set wait states
            RTS                             ;

;*******************************************************************************
;   TEMPERATURE MONITOR -- AD7888.ASM
;   DIN=PD5(STD) DOUT=PD4(SRD) SCLK=PD3(SCK) CS=PD2(SFS)
;*******************************************************************************
TEMP_READ   MOVE    #$0,X0                  ;
            MOVE    #>$FFF,X1               ;
            MOVE    #TCCD,R3                ;

            DO      #$8,END_ADC8            ;
            CLR     A                       ;
            MOVE    X0,A0                   ;
            INC     A                       ;
            NOP                             ;
            MOVE    A0,X0                   ;
            BCLR    #$3,A0                  ;
            ASL     #$3,A,A                 ;
            NOP                             ;
            BSET    #$2,A0                  ;
            JSR     ADC_RD                  ;
            AND     X1,A                    ;
            NOP                             ;
            MOVE    A1,X:(R3)+              ;
END_ADC8    RTS                             ;

SEND_TEMP   MOVE    #TCCD,R3                ;
            DO      #$8,END_ST              ;
            MOVE    X:(R3)+,A1              ;
            JSR     WRITE16                 ;
            NOP                             ;
END_ST      RTS                             ;

ADC_RD      BCLR    #$2,X:PDRD              ; assert /CS
            DO      #$10,END_ADC10          ; set up 16 clocks
            JSET    #$7,A0,WD1              ;
WD0         BCLR    #$5,X:PDRD              ; STD=0
            JCLR    #$7,A0,WCLKS            ;
WD1         BSET    #$5,X:PDRD              ; STD=1
WCLKS       BCLR    #$3,X:PDRD              ;
            REP     #$28                    ; was $F
            NOP                             ;
            BSET    #$3,X:PDRD              ;
            REP     #$28                    ;
            NOP                             ;
            ASL     A                       ;
            JCLR    #$4,X:PDRD,RD0          ;
RD1         BSET    #$0,A1                  ; SRD=1
RD0         NOP                             ; SRD=0
END_ADC10   BSET    #$2,X:PDRD              ; deassert /CS
            BCLR    #$3,X:PDRD              ;
            RTS                             ;

TEMP_PID    MOVE    #>TSET,R3               ; e0 = Tset-Tccd (DN)
            MOVE    #>E0,R4                 ; 1 LSB = 0.102 degC
            MOVE    #>(TCCD-TSET),N3        ;
            DO      #$8,END_ERR             ; TSET  $620 = -100 degC
            MOVE    X:(R3+N3),X0            ; 
            MOVE    X:(R3)+,A               ;
            SUB     X0,A                    ; KP = 20 mA/degC
            NOP                             ; KI = 2 mA/degC, T=10sec
            MOVE    A1,X:(R4)+              ; KH = 20 DN/mA
END_ERR     NOP                             ;
PID_EQN     MOVE    #>E0,R3                 ;
            MOVE    #>H0,R4                 ;
            MOVE    #>(E1-E0),N3            ;
            MOVE    #>(H1-H0),N4            ;
            DO      #$8,END_H               ;
            MOVE    X:(R3),A                ; h0=h1+g0*(e0-e1)+g1*(e0+e1)
            MOVE    X:(R3+N3),X0            ;
            SUB     X0,A                    ;
            MOVE    #>$28,X1                ; g0=$28 g1=$2
            MOVE    A1,Y1                   ;
            MPY     X1,Y1,A                 ;
            ASL     #$17,A,A                ;
            NOP                             ;
            MOVE    A1,Y0                   ; P term = g0*(e0-e1)  in A1
            MOVE    X:(R3+N3),X0            ;
            MOVE    X:(R3),A                ;
            NOP                             ;
            MOVE    A1,X:(R3+N3)            ; e1 = e0
            ADD     X0,A                    ;
            MOVE    #>2,X1                  ; g1=$2
            MOVE    A1,Y1                   ;
            MPY     X1,Y1,A                 ;
            ASL     #$17,A,A                ;
            NOP                             ; I term = g1*(e0+e1)  in A1
            ADD     Y0,A                    ; g1=$2
            MOVE    X:(R4+N4),X0            ;
            ADD     X0,A                    ;
            NOP                             ;
            MOVE    A1,X:(R4)               ;
            MOVE    X:(R4),A                ; h0
LCHECK      MOVE    #>$0,X0                 ; if(h0<0) h0=0;
            CMP     X0,A                    ;
            JGE     HCHECK                  ;
            MOVE    X0,A                    ;
HCHECK      MOVE    #>$FA0,X0               ; if(h0>100mA)h0=100mA;
            CMP     X0,A                    ; $FA0 = 20*200 $1200=20*230
            JLE     H0_SET                  ; $7D0 = 20*100
            MOVE    X0,A                    ; 20DN = 1 mA
            NOP                             ;
H0_SET      MOVE    A1,X:(R4+N4)            ; h1 = h0
            MOVE    A1,X:(R4)+              ;
            MOVE    X:(R3)+,X0              ; update R3
END_H       NOP                             ;
H_SET       MOVE    #>H0,R3                 ;
            MOVE    #>HEAT,R4               ;
            MOVE    #>DZ,X0                 ;
            DO      #$8,END_HSET            ;
            MOVE    X:(R3)+,A               ;
            ADD     X0,A                    ;
            NOP                             ;
            MOVE    A1,X:(R4)+              ;
END_HSET    NOP                             ;
    IF (SYS==30||SYS==31||SYS==32)          ; IMACS-only
            MOVE    #>DZ,X1                 ; PID loop reads all 8 temp sensors.
            MOVE    X:(HEAT+1),X0           ; Pairs of chips are A+B C+D F+E H+G
            MOVE    X0,X:HEAT               ; Sense using B, D, E, G (warm)
            MOVE    X1,X:(HEAT+1)           ; Heat  using A, C, F, H (cold)
            MOVE    X:(HEAT+3),X0           ;
            MOVE    X0,X:(HEAT+2)           ; Pairs are strapped together, one
            MOVE    X1,X:(HEAT+3)           ; is warmer than the other.
            MOVE    X:(HEAT+4),X0           ;
            MOVE    X0,X:(HEAT+5)           ;
            MOVE    X1,X:(HEAT+4)           ;
            MOVE    X:(HEAT+6),X0           ;
            MOVE    X0,X:(HEAT+7)           ;
            MOVE    X1,X:(HEAT+6)           ;
    ENDIF                                   ;
    IF (SYS<30)                             ;
            MOVE    X:(HEAT+INIT_TCH),X0    ; Sense using TCH
            MOVE    X0,X:(HEAT+0)           ; Same value to A,B,C,D heaters
            MOVE    X0,X:(HEAT+1)           ; 
            MOVE    X0,X:(HEAT+2)           ; 
            MOVE    X0,X:(HEAT+3)           ; 
    ENDIF                                   ;

;   IF (SYS==20||SYS==21)                   ; LDSS-only
;           MOVE    #>DZ,X1                 ; PID loop reads all 4 temp sensors.
;           MOVE    X:(HEAT+3),X0           ; All 4 sensors are in same Al block
;           NOP
;           MOVE    X0,X:(HEAT+1)           ; Sense using D only
;           MOVE    X0,X:(HEAT+2)           ; Heat using A, B, C, D
;           MOVE    X0,X:(HEAT+0)           ;
;   ENDIF                                   ;
;   IF (SYS==8||SYS==9||SYS==10||SYS==11||SYS==13)   ; SITE2k, WF4K, PFS4K
;           MOVE    #>DZ,X1                 ; PID loop reads both temp sensors
;           MOVE    X:(HEAT+0),X0           ; Both sensors are in same Al block
;           NOP                             ; Sense using A
;           MOVE    X0,X:(HEAT+1)           ; Heat using A,B
;   ENDIF                                   ;
;   IF (SYS==1||SYS==2||SYS==3)                     ; LAB, MIKE_RED, MIKE-BLUE
;           MOVE    #>DZ,X1                 ; PID loop reads both temp sensors
;           MOVE    X:(HEAT+0),X0           ; Sensor A=platen, B=on-chip 
;           NOP                             ; Sense using A
;           MOVE    X0,X:(HEAT+1)           ; Heat using A,B
;   ENDIF                                   ;
            JSR     SET_HEAT                ; HEAT(DN) = 20 * u(mA) + DZ
            RTS

;*******************************************************************************
;   MILLISECOND AND MICROSECOND TIMER MODULE
;*******************************************************************************
U_TIMER     BSET    #$0,X:TCSR0             ; start timer
            BTST    #$0,X:TCSR0             ; delay for flag update
            JCLR    #$15,X:TCSR0,*          ; wait for TCF flag
            BCLR    #$0,X:TCSR0             ; stop timer, clear flag
            RTS                             ; flags update during RTS

M_TIMER     BSET    #$0,X:TCSR1             ; start timer
            BTST    #$0,X:TCSR0             ; delay for flag update
            JCLR    #$15,X:TCSR1,*          ; wait for TCF flag
            BCLR    #$0,X:TCSR1             ; stop timer, clear flag
            RTS                             ; flags update during RTS

;*******************************************************************************
;   SIGNAL-PROCESSING GAIN & DUAL_SLOPE MODULEs
;*******************************************************************************
SET_GAIN    JSET    #$0,X:GAIN,HI_GAIN      ;
LO_GAIN     BCLR    #$1,X:PDRB              ; set gain=0
HI_GAIN     JCLR    #$0,X:GAIN,END_GAIN     ;
            BSET    #$1,X:PDRB              ; set gain=1
END_GAIN    RTS                             ;


SET_USEC    MOVEP   X:USEC,X:TCPR0          ; timer compare value
END_USEC    RTS                             ;

;*******************************************************************************
;   SELECT OUTPUT CHANNEL (IE OUTPUT AMPLIFIER)
;*******************************************************************************
    IF (RLMODE==1)                         ;
SET_OPCH    MOVE    X:OPCH,A                ;
            ADD     #>PIX,A                 ;
            NOP                             ;
            MOVE    A1,N5                   ;
END_AMP     RTS                             ;
    ENDIF                                   ;

    IF (RLMODE==2)                         ;
    IF (SYS!=21)                            ;
SET_OPCH    MOVE    #PIX,N5                 ;
END_AMP     RTS                             ;
    ENDIF                                   ;

    IF (SYS==21)                            ; LDSS 2amp
SET_OPCH    MOVE    #(PIX+2),N5             ;
END_AMP     RTS                             ;
    ENDIF                                   ;
    ENDIF                                   ;

    IF (RLMODE==3)                         ;
SET_OPCH    MOVE    #PIX,N5                 ;
END_AMP     RTS                             ;
    ENDIF                                   ;

SET_SCLKS   JCLR    #$0,X:HCLKS,S_RIGHT     ; 0x1=left 0x2=right
S_LEFT      MOVE    #>SCLKS_L,A0            ; 0x3=both
            MOVE    #>SCLKS_FLL,A1          ;
            MOVE    #>INT_L,R1              ;
S_RIGHT     JCLR    #$1,X:HCLKS,S_BOTH      ;
            MOVE    #>SCLKS_R,A0            ;
            MOVE    #>SCLKS_FLR,A1          ;
            MOVE    #>INT_R,R1              ;
S_BOTH      JCLR    #$1,X:HCLKS,END_HD      ;
            JCLR    #$0,X:HCLKS,END_HD      ;
            MOVE    #>SCLKS_B,A0            ;
            MOVE    #>SCLKS_FLB,A1          ;
            MOVE    #>INT_L,R1              ;
            NOP                             ;
END_HD      MOVE    A0,X:SCLKS              ;
            MOVE    A1,X:SCLKS_FL           ;
            MOVE    R1,X:INT_X              ;
END_SCLKS   RTS                             ;

;*******************************************************************************
;   CMD.ASM -- ROUTINE TO INTERPRET AN 8-BIT COMMAND + COMPLEMENT
;*******************************************************************************
; Each command word is sent as two bytes -- the LSB has the command
; and the MSB has the complement.

CMD_FIX     MOVE    X:CMD,A                 ; get CMD word
            JSR     WRITE16                 ; ack the CMD word
CHKCMP      AND     #>$FF,A                 ; extract cmd[7..0]
            MOVE    A1,X1                   ; and put in X1
            MOVE    X:CMD,A                 ; extract cmd[15..8]
            LSR     #$8,A                   ; complement
            NOT     A   #>$1,B              ; and put in A1
            AND     #>$FF,A                 ;
            ASL     X1,B,B                  ;
            CMP     X1,A                    ; compare X1 and A1
            JEQ     CMD_OK                  ;
CMD_NG      CLR     B                       ; cmd word no good
            NOP                             ;
CMD_OK      MOVE    B1,X:CMD                ; cmd word OK
            NOP                             ;
END_CMD     RTS                             ;


;*****************************************************************************
;   INITIALIZATION ROUTINES & DATA
;*****************************************************************************
INIT_MEM    CLR     A                       ; Clear out XMEM,YMEM
            MOVE    #$0,R2                  ; #$800=2048
            DO      #$40,END_BLXY           ;
            MOVE    A,X:(R2)                ;
            MOVE    A,Y:(R2)+               ;
END_BLXY    NOP                             ;
INIT0       MOVE    #>INIT_NROWS,X0         ; Set intial values
            MOVE    X0,X:NROWS              ;
            MOVE    #>INIT_NCOLS,X0         ;
            MOVE    X0,X:NCOLS              ;
            MOVE    #>INIT_NFT,X0           ;
            MOVE    X0,X:NFT                ;
            MOVE    #>INIT_NFLUSH,X0        ;
            MOVE    X0,X:NFLUSH             ;
            MOVE    #>INIT_NCH,X0           ;
            MOVE    X0,X:NCH                ;
            MOVE    #>INIT_VBIN,X0          ;
            MOVE    X0,X:VBIN               ;
            MOVE    #>INIT_HBIN,X0          ;
            MOVE    X0,X:HBIN               ;
            MOVE    #>INIT_VSKIP,X0         ;
            MOVE    X0,X:VSKIP              ;
            MOVE    #>INIT_HSKIP,X0         ;
            MOVE    X0,X:HSKIP              ;
            MOVE    #>INIT_EXPTIME,X0       ;
            MOVE    X0,X:EXP_TIME           ;
            MOVE    #>INIT_GAIN,X0          ;
            MOVE    X0,X:GAIN               ;
            MOVE    #>INIT_USEC,X0          ;
            MOVE    X0,X:USEC               ;
            MOVE    #>INIT_OPCH,X0          ;
            MOVE    X0,X:OPCH               ;
            MOVE    #>INIT_HCLKS,X0         ;
            MOVE    X0,X:HCLKS              ;
            MOVE    #>INIT_PID,X0           ;
            MOVE    X0,X:PID                ;
;INIT1      MOVE    #>$436,X0               ; Temp setpoint 
;           MOVE    #TSET,R4                ; $436 = -150C
;           DO      #$8,END_IN1             ;
;           MOVE    X0,X:(R4)+              ;
;END_IN1    NOP                             ;
;INIT2      MOVE    #(DZ+0000),X0           ;
;           MOVE    #HEAT,R4                ;
;           DO      #$8,END_IN2             ;
;           MOVE    X0,X:(R4)+              ;
;END_IN2    NOP                             ;
;INIT3      MOVE    #(DZ+0000),X0           ;
;           MOVE    #VBIAS,R4               ;
;           DO      #$40,END_IN3            ;
;           MOVE    X0,X:(R4)+              ;
;END_IN3    NOP                             ;
;INIT4      MOVE    #(DZ+0000),X0           ;
;           MOVE    #VCLK,R4               ;
;           DO      #$80,END_IN4            ;
;           MOVE    X0,X:(R4)+              ;
;END_IN4    NOP                             ;
;XCOPYA     MOVE    #A_LOAD,R3              ; copy voltages from P:LOAD
;           MOVE    #VBIAS,R4               ; to X:RUN location
;           DO      #$18,END_CPA            ;
;           MOVE    P:(R3)+,X0              ;
;           MOVE    X0,X:(R4)+              ;
;END_CPA    NOP                             ;
;XCOPYB     MOVE    #B_LOAD,R3              ; copy sequences from P:LOAD
;           MOVE    #MPP,R4                 ; to X:RUN location
;           DO      #$80,END_CPB            ;
;           MOVE    P:(R3)+,X0              ;
;           MOVE    X0,X:(R4)+              ;
;END_CPB    NOP                             ;
            RTS                             ;


            END

