;*******************************************************************************
;   DEV_BNC.ASM -- DEVICE SPECIFIC OPTIONS, CLOCKING SEQUENCES & VOLTAGES
;*******************************************************************************
    NOLIST
;******* FOR B&C (E2V CCD42-10) ************************************************
RLMODE          EQU     $1              ; $1=normal
RDMODE          EQU     $0              ; $0=normal
HOLD_P          EQU     $4E20           ; P  clock timing ($7D00=400 us)
HOLD_S          EQU     $0006           ; S  clock timing
HOLD_FT         EQU     $4E20           ; FT clock timing ($4E20=250 us)
HOLD_FL         EQU     $4E20           ; FL clock timimg
HOLD_PL         EQU     $1F40           ; PL settling ($1F40=100us)
HOLD_RG         EQU     $0008           ; RG timing
HOLD_FF         EQU     $0032           ; FF clock timimg
HOLD_IPC        EQU     $0000           ; IPC clock timing
HOLD_SIG        EQU     $0010           ; SIG settling time
HOLD_ADC        EQU     $0010           ; pre-sample settling
INIT_NROWS      EQU     $223            ; $220=515+32
INIT_NCOLS      EQU     $852            ; $852=2048+50+32
INIT_NFT        EQU     $203            ; $203=515
INIT_NFLUSH     EQU     $864            ; $864=2048+100
INIT_NCH        EQU     $1              ;
INIT_VBIN       EQU     $1              ;
INIT_HBIN       EQU     $1              ;
INIT_VSKIP      EQU     $0              ;
INIT_HSKIP      EQU     $32             ; $32=50
INIT_GAIN       EQU     $1              ; 0=LOW 1=HIGH
INIT_USEC       EQU     $C8             ;
INIT_OPCH       EQU     $0              ;
INIT_HCLKS      EQU     $2              ;
INIT_TCH        EQU     $0              ; 0=TEMP_A, 1=TEMP_B
INIT_PID        EQU     $1              ;
INIT_EXPTIME    EQU     $3E8            ;

;*******************************************************************************
;   INITIALIZE VOLTAGE VALUES (IN X MEMORY)
;*******************************************************************************
            ORG     X:TSET              ;
            DC      $000620             ; $682 = - 90 degC
            DC      $000620             ; $651 = - 95
            DC      $000000             ; $620 = -100
            DC      $000000             ; $5EF = -105
            DC      $000000             ; $5BE = -110
            DC      $000000             ; $58C = -115
            DC      $000000             ; $55C = -120
            DC      $000000             ; $52B = -125

            ORG     X:VBIAS
            DC      (DZ-0040)           ; OFF_A (5 mV/DN)
            DC      (DZ-0000)           ; ADJ_A
            DC      (DZ-1500)           ; OG_A=OG1
            DC      (DZ+0000)           ; ABG_A
            DC      (DZ+0750)           ; RD_A (10 mV/DN)
            DC      (DZ+0000)           ; SP_A
            DC      (DZ+1350)           ; OFD_A=DD
            DC      (DZ+1950)           ; OD_A
            DC      (DZ+0000)           ; OFF_B (5 mV/DN)
            DC      (DZ-0000)           ; ADJ_B
            DC      (DZ-1300)           ; OG_B=OG2
            DC      (DZ+0000)           ; ABG_B
            DC      (DZ+0000)           ; RD_B (10 mV/DN)
            DC      (DZ+0000)           ; SP_B
            DC      (DZ+0000)           ; OFD_B
            DC      (DZ+0000)           ; OD_B

            ORG     X:VCLK
            DC      (DZ-0000)           ; IPC_A- (5 mV/DN)
            DC      (DZ+1000)           ; IPC_A+
            DC      (DZ-0000)           ; IPC_B-
            DC      (DZ+1000)           ; IPC_B+
            DC      (DZ-1900)           ; RG_A-
            DC      (DZ+0300)           ; RG_A+
            DC      (DZ+0000)           ; RG_B-
            DC      (DZ+0000)           ; RG_B+
            DC      (DZ-1900)           ; SW_A-
            DC      (DZ+0300)           ; SW_A+
            DC      (DZ-0000)           ; SW_B-
            DC      (DZ+0000)           ; SW_B+
            DC      (DZ-2000)           ; TG_A- = DG_A-
            DC      (DZ+0000)           ; TG_A+ = DG_A+
            DC      (DZ-0000)           ; TG_B-
            DC      (DZ+0000)           ; TG_B+
            DC      (DZ-1900)           ; S_A-
            DC      (DZ+0300)           ; S_A+
            DC      (DZ-0000)           ; S_B-
            DC      (DZ+0000)           ; S_B+
            DC      (DZ-2000)           ; P1_A-
            DC      (DZ+0300)           ; P1_A+
            DC      (DZ-2000)           ; P2_A-
            DC      (DZ+0300)           ; P2_A+
            DC      (DZ-2000)           ; P3_A-
            DC      (DZ+0300)           ; P3_A+
            DC      (DZ-0000)           ; P1_B-
            DC      (DZ+0000)           ; P1_B+
            DC      (DZ-0000)           ; P2_B-
            DC      (DZ+0000)           ; P2_B+
            DC      (DZ-0000)           ; P3_B-
            DC      (DZ+0000)           ; P3_B+

;*******************************************************************************
;   INITIALIZE SEQ FRAGMENTS (IN X MEMORY)
;*******************************************************************************
;        S2B   _______________  ________________ S1B
;        S3B   ______________ || _______________ S3A
;        SW    _____________ |||| ______________ S2A
;        TG    ____________ |||||| _____________ S1A
;        P1A   ___________ |||||||| ____________ RG
;        P2A   __________ |||||||||| ___________ IPC
;        P3A   _________ |||||||||||| __________ FINT+
;        P1B   ________ |||||||||||||| _________ FINT-
;        P2B   _______ |||||||||||||||| ________ FRST
;        P3B   ______ |||||||||||||||||| _______ CONVST
;                    ||||||||||||||||||||

    IF (RDMODE==$0)                 ; E2V CCD42-10, non-MPP
            ORG X:MPP               ; reset/hold state
            DC  %000000000000011011000011

            ORG X:IPCLKS            ; input clamp
            DC  %000000000000011011010011
            DC  %000000000000011011000011

            ORG X:PCLKS_RD          ; parallel transfer P3-P1-P2
            DC  %000010010000011011000011
            DC  %000010110100011011000011
            DC  %000000100100011011000011
            DC  %000001101100011011000011
            DC  %000001001000011011000011
            DC  %000000000000011011000011

            ORG X:PCLKS_FL          ; parallel flush P3-P1-P2
            DC  %000010010000011011000011
            DC  %000010110100011011000011
            DC  %000000100100011011000011
            DC  %000001101100011011000011
            DC  %000001001000011011000011
            DC  %000000000000011011000011

            ORG X:INT_L             ; reset and integrate left
            DC  %000000000000010010100011   ; RG ON  FRST ON
            DC  %000000000000010010000011   ; RG OFF
            DC  %000000000001010010000001   ; FRST OFF SW ON
            DC  %000000000001010010001001   ; FINT+ ON
            DC  %000000000001010010000001   ; FINT+ OFF
            DC  %000000000000010010000001   ; SW OFF
            DC  %000000000000010010000101   ; FINT- ON
            DC  %000000000000010010000001   ; FINT- OFF
            DC  %000000000000010010000000   ; /CONVST ON
            DC  %000000000000010010000001   ; /CONVST OFF
            DC  %000000000000010010000011   ; FRST ON

            ORG X:INT_R             ; reset and integrate right
            DC  %000000000000001001100011   ; RG ON  FRST ON
            DC  %000000000000001001000011   ; RG OFF
            DC  %000000000001001001000001   ; FRST OFF SW ON
            DC  %000000000001001001001001   ; FINT+ ON
            DC  %000000000001001001000001   ; FINT+ OFF
            DC  %000000000000001001000001   ; SW OFF
            DC  %000000000000001001000101   ; FINT- ON
            DC  %000000000000001001000001   ; FINT- OFF
            DC  %000000000000001001000000   ; /CONVST ON
            DC  %000000000000001001000001   ; /CONVST OFF
            DC  %000000000000001001000011   ; FRST ON

            ORG X:SCLKS_L           ; serial shift (left=A) S2-S1-S3-S2
            DC  %000000000001011011000011
            DC  %000000000001001001000011
            DC  %000000000001101101000011
            DC  %000000000001100100000011
            DC  %000000000001110110000011
            DC  %000000000001010010000011

            ORG X:SCLKS_R           ; serial shift (right=B) S1-S2-S3-S1
            DC  %000000000001011011000011
            DC  %000000000001010010000011
            DC  %000000000001110110000011
            DC  %000000000001100100000011
            DC  %000000000001101101000011
            DC  %000000000001001001000011

            ORG X:SCLKS_FLL         ; serial flush (left=A) S2-S1-S3-S2
            DC  %000000000001011011100011
            DC  %000000000001001001100011
            DC  %000000000001101101100011
            DC  %000000000001100100100011
            DC  %000000000001110110100011
            DC  %000000000000010010000011

            ORG X:SCLKS_FLR         ; serial flush (right=B) S1-S2-S3-S1
            DC  %000000000001011011100011
            DC  %000000000001010010100011
            DC  %000000000001110110100011
            DC  %000000000001100100100011
            DC  %000000000001101101100011
            DC  %000000000000001001000011
    ENDIF
