;*******************************************************************************
;   DEV_LAB.ASM -- DEVICE SPECIFIC OPTIONS, CLOCKING SEQUENCES & VOLTAGES
;*******************************************************************************
    NOLIST
;******* LAB SYSTEM (VARIOUS CCD) **********************************************
RLMODE          EQU     $1              ; $1=2-CH $2=4-CH $3=8-CH
RDMODE          EQU     $0              ; $0=normal $1=alternate
HOLD_P          EQU     $20A            ; P  clock timing ($20A=40 us)
HOLD_S          EQU     $6              ; S  clock timing
HOLD_FT         EQU     $7C             ; FT clock timing ($7C=10 us)
HOLD_FL         EQU     $7C             ; FL clock timimg
HOLD_PL         EQU     $1F40           ; pre-line settling ($1F40=100us)
HOLD_RG         EQU     $6              ; RG timing
HOLD_FF         EQU     $32             ; FF clock timimg
HOLD_IPC        EQU     $10             ; IPC clock timing
HOLD_SIG        EQU     $10             ; SIG settling time
HOLD_ADC        EQU     $10             ; pre-sample settling
INIT_NROWS      EQU     $213            ; $213=(1030/2)+16
INIT_NCOLS      EQU     $238            ; $238=(1072/2)+32
INIT_NFT        EQU     $406            ; $406=1030
INIT_NFLUSH     EQU     $430            ; $430=1072
INIT_NCH        EQU     $1              ;
INIT_VBIN       EQU     $2              ;
INIT_HBIN       EQU     $2              ;
INIT_VSKIP      EQU     $0              ;
INIT_HSKIP      EQU     $0              ;
INIT_GAIN       EQU     $1              ; 0=LOW 1=HIGH
INIT_USEC       EQU     $C8             ;
INIT_OPCH       EQU     $0              ; 0=CH_A 1=CH_B
INIT_HCLKS      EQU     $1              ; 1=LEFT 2=RIGHT 3=BOTH
INIT_TCH        EQU     $0              ; 0=TEMP_A, 1=TEMP_B
INIT_PID        EQU     $0              ; FLAG $0=OFF $1=ON
INIT_EXPTIME    EQU     $3E8            ;

;*******************************************************************************
;   INITIALIZE VOLTAGE VALUES (IN X MEMORY)
;*******************************************************************************
            ORG     X:TSET              ; TEMP SETPOINT
            DC      $0005BE             ; $682 = - 90 degC
            DC      $0005BE             ; $651 = - 95
            DC      $000000             ; $620 = -100
            DC      $000000             ; $5EF = -105
            DC      $000000             ; $5BE = -110
            DC      $000000             ; $58C = -115
            DC      $000000             ; $55C = -120
            DC      $000000             ; $52B = -125

            ORG     X:VBIAS             ; BIAS VOLTAGES
            DC      (DZ-0040)           ; OFF_A (5 mV/DN)
            DC      (DZ-0000)           ; ADJ_A
            DC      (DZ-1200)           ; OG1_A
            DC      (DZ-0000)           ; OG2_A
            DC      (DZ+0800)           ; RD_A (10 mV/DN)
            DC      (DZ+0000)           ; SP_A
            DC      (DZ+0000)           ; OFD_A
            DC      (DZ+2000)           ; OD_A
            DC      (DZ-0040)           ; OFF_B (5 mV/DN)
            DC      (DZ-0000)           ; ADJ_B
            DC      (DZ-1200)           ; OG1_B
            DC      (DZ-0000)           ; OG2_B
            DC      (DZ+0800)           ; RD_B (10 mV/DN)
            DC      (DZ+0000)           ; SP_B
            DC      (DZ+0000)           ; OFD_B
            DC      (DZ+2000)           ; OD_B

            ORG     X:VCLK              ; CLOCK VOLTAGES
            DC      (DZ-0000)           ; IPC_A- (5 mV/DN)
            DC      (DZ+1000)           ; IPC_A+
            DC      (DZ-0000)           ; IPC_B-
            DC      (DZ+1000)           ; IPC_B+
            DC      (DZ-1800)           ; RG_A-
            DC      (DZ+0600)           ; RG_A+
            DC      (DZ-1800)           ; RG_B-
            DC      (DZ+0600)           ; RG_B+
            DC      (DZ-1600)           ; SW_A-
            DC      (DZ+0400)           ; SW_A+
            DC      (DZ-1600)           ; SW_B-
            DC      (DZ+0400)           ; SW_B+
            DC      (DZ-1800)           ; TG_A-
            DC      (DZ+0600)           ; TG_A+
            DC      (DZ-1800)           ; TG_B-
            DC      (DZ+0600)           ; TG_B+
            DC      (DZ-1600)           ; S_A-
            DC      (DZ+0400)           ; S_A+
            DC      (DZ-1600)           ; S_B-
            DC      (DZ+0400)           ; S_B+
            DC      (DZ-1800)           ; P1_A-
            DC      (DZ+0600)           ; P1_A+
            DC      (DZ-1800)           ; P2_A-
            DC      (DZ+0600)           ; P2_A+
            DC      (DZ-1800)           ; P3_A-
            DC      (DZ+0600)           ; P3_A+
            DC      (DZ-1800)           ; P1_B-
            DC      (DZ+0600)           ; P1_B+
            DC      (DZ-1800)           ; P2_B-
            DC      (DZ+0600)           ; P2_B+
            DC      (DZ-1800)           ; P3_B-
            DC      (DZ+0600)           ; P3_B+

;*******************************************************************************
;   INITIALIZE SEQ FRAGMENTS (IN X MEMORY)
;*******************************************************************************
;        S2B   _______________  ________________ S1B
;        S3B   ______________ || _______________ S3A
;        SW    _____________ |||| ______________ S2A
;        TG    ____________ |||||| _____________ S1A
;        P1A   ___________ |||||||| ____________ RG
;        P2A   __________ |||||||||| ___________ IPC
;        P3A   _________ |||||||||||| __________ FINT+
;        P1B   ________ |||||||||||||| _________ FINT-
;        P2B   _______ |||||||||||||||| ________ FRST
;        P3B   ______ |||||||||||||||||| _______ CONVST
;                    ||||||||||||||||||||

    IF (RDMODE==$0)                 ; E2V CCD47-20, MPP, implant=phase2
            ORG X:MPP               ; reset/hold state
            DC  %000000000000011011000011

            ORG X:IPCLKS            ; input clamp
            DC  %000000000000011011010011
            DC  %000000000000011011000011

            ORG X:PCLKS_FT          ; frame transfer P1-P2-P3-P1 non-MPP
            DC  %000001101100011011000011
            DC  %000001001000011011000011
            DC  %000011011000011011000011
            DC  %000010010000011011000011
            DC  %000010110100011011000011
            DC  %000000100100011011000011

            ORG X:PCLKS_RD          ; parallel transfer P1-P2-P3
            DC  %000000000100011011000011
            DC  %000000001100011011000011
            DC  %000000001000011011000011
            DC  %000000011000011011000011
            DC  %000000010000011011000011
            DC  %000000000000011011000011

            ORG X:PCLKS_FL          ; parallel flush P1-P2-P3
            DC  %000000000100011011000011
            DC  %000000001100011011000011
            DC  %000000001000011011000011
            DC  %000000011000011011000011
            DC  %000000010000011011000011
            DC  %000000000000011011000011

            ORG X:INT_L             ; reset and integrate left
            DC  %000000000000001001100011   ; RG ON  FRST ON
            DC  %000000000000001001000011   ; RG OFF
            DC  %000000000000001001000001   ; FRST OFF
            DC  %000000000000001001001001   ; FINT+ ON
            DC  %000000000000001001000001   ; FINT+ OFF

            DC  %000000000000001001000101   ; FINT- ON
            DC  %000000000000001001000001   ; FINT- OFF
            DC  %000000000000001001000000   ; /CONVST ON
            DC  %000000000000001001000001   ; /CONVST OFF
            DC  %000000000000001001100011   ; FRST ON RG ON

; orig      ORG X:INT_L             ; reset and integrate left
;           DC  %000001001000010010100011   ; RG ON  FRST ON
;           DC  %000001001000010010000011   ; RG OFF
;           DC  %000001001001010010000001   ; FRST OFF SW ON
;           DC  %000001001001010010001001   ; FINT+ ON
;           DC  %000001001001010010000001   ; FINT+ OFF
;           DC  %000001001000010010000001   ; SW OFF
;           DC  %000001001000010010000101   ; FINT- ON
;           DC  %000001001000010010000001   ; FINT- OFF
;           DC  %000001001000010010000000   ; /CONVST ON
;           DC  %000001001000010010000001   ; /CONVST OFF
;           DC  %000001001000010010000011   ; FRST ON  [note: IPC !!]

;           ORG X:INT_R             ; reset and integrate right
;           DC  %000001001000001001100011   ; RG ON  FRST ON
;           DC  %000001001000001001000011   ; RG OFF
;           DC  %000001001001001001000001   ; FRST OFF SW ON
;           DC  %000001001001001001001001   ; FINT+ ON
;           DC  %000001001001001001000001   ; FINT+ OFF
;           DC  %000001001000001001000001   ; SW OFF
;           DC  %000001001000001001000101   ; FINT- ON
;           DC  %000001001000001001000001   ; FINT- OFF
;           DC  %000001001000001001000000   ; /CONVST ON
;           DC  %000001001000001001000001   ; /CONVST OFF
;           DC  %000001001000001001000011   ; FRST ON

;        S2B   _______________  ________________ S1B
;        S3B   ______________ || _______________ S3A
;        SW    _____________ |||| ______________ S2A
;        TG    ____________ |||||| _____________ S1A
;        P1A   ___________ |||||||| ____________ RG
;        P2A   __________ |||||||||| ___________ IPC
;        P3A   _________ |||||||||||| __________ FINT+
;        P1B   ________ |||||||||||||| _________ FINT-
;        P2B   _______ |||||||||||||||| ________ FRST
;        P3B   ______ |||||||||||||||||| _______ CONVST
;                    ||||||||||||||||||||

            ORG X:SCLKS_L           ; serial shift (left=A) S1-S3-S2-S1
            DC  %000000000000101101000011
            DC  %000000000000100100000011
            DC  %000000000000110110000011
            DC  %000000000000010010000011
            DC  %000000000000011011000011
            DC  %000000000000001001000011

            ORG X:SCLKS_R           ; serial shift (right=B) S1-S2-S3-S1
            DC  %000000000000011011000011
            DC  %000000000000010010000011
            DC  %000000000000110110000011
            DC  %000000000000100100000011
            DC  %000000000000101101000011
            DC  %000000000000001001000011

            ORG X:SCLKS_B           ; serial shift both
            DC  %000000000000011101000011   ; (left=A)  S1-S3-S2-S1
            DC  %000000000000010100000011   ; (right=B) S1-S2-S3-S1 
            DC  %000000000000110110000011
            DC  %000000000000100010000011
            DC  %000000000000101011000011
            DC  %000000000000001001000011

            ORG X:SCLKS_FLL         ; serial flush (left=A) S1-S3-S2-S1
            DC  %000000000000101101100011
            DC  %000000000000100100100011
            DC  %000000000000110110100011
            DC  %000000000000010010100011
            DC  %000000000000011011100011
            DC  %000000000000001001000011

            ORG X:SCLKS_FLR         ; serial flush (right=B) S1-S2-S3-S1
            DC  %000000000000011011100011
            DC  %000000000000010010100011
            DC  %000000000000110110100011
            DC  %000000000000100100100011
            DC  %000000000000101101100011
            DC  %000000000000001001000011

            ORG X:SCLKS_FLB         ; serial flush both
            DC  %000000000000011101100011   ; (left=A)  S1-S3-S2-S1
            DC  %000000000000010100100011   ; (right=B) S1-S2-S3-S1
            DC  %000000000000110110100011
            DC  %000000000000100010100011
            DC  %000000000000101011100011
            DC  %000000000000001001000011

            ORG X:SCLKS_FF          ; serial fast flush DG
            DC  %000000000010011011100011 ; SCLKS ON RG ON DG ON
            DC  %000000000010000000100011 ; SCLKS OFF
            DC  %000000000000000000100011 ; DG OFF
            DC  %000000000000011011100011 ; SCLKS ON
            DC  %000000000000011011000011 ; RG OFF
            DC  %000000000000011011000011   ; dummy code

            ORG X:PCLKS_FWD          ; parallel shuffle P2-P3-P1-P2
            DC  %000000000000011011000011   ; P2-P1-P3-P2
            DC  %000000000000011011000011
            DC  %000000000000011011000011
            DC  %000000000000011011000011
            DC  %000000000000011011000011
            DC  %000000000000011011000011

            ORG X:PCLKS_REV          ; parallel shuffle P2-P1-P3-P2
            DC  %000000000000011011000011   ; P2-P3-P1-P2
            DC  %000000000000011011000011
            DC  %000000000000011011000011
            DC  %000000000000011011000011
            DC  %000000000000011011000011
            DC  %000000000000011011000011
    ENDIF

    IF (RDMODE==$1)                 ; E2V CCD47-20, non-MPP (implant=phase2)
            ORG X:MPP               ; reset/hold state
            DC  %000000100100011011000011

            ORG X:IPCLKS            ; input clamp
            DC  %000000100100011011010011
            DC  %000000100100011011000011

            ORG X:PCLKS_FT          ; parallel transfer P1-P2-P3-P1
            DC  %000001101100011011000011
            DC  %000001001000011011000011
            DC  %000011011000011011000011
            DC  %000010010000011011000011
            DC  %000010110100011011000011
            DC  %000000100100011011000011

            ORG X:PCLKS_RD          ; parallel transfer P1-P2-P3-P1
            DC  %000000101100011011000011
            DC  %000000101000011011000011
            DC  %000000111000011011000011
            DC  %000000110000011011000011
            DC  %000000110100011011000011
            DC  %000000100100011011000011

            ORG X:PCLKS_FL          ; parallel flush P1-P2-P3
            DC  %000000101100011011000011
            DC  %000000101000011011000011
            DC  %000000111000011011000011
            DC  %000000110000011011000011
            DC  %000000110100011011000011
            DC  %000000100100011011000011

    ENDIF
