;*******************************************************************************
;   DEV_*.ASM -- DEVICE SPECIFIC OPTIONS, CLOCKING SEQUENCES & VOLTAGES
;*******************************************************************************
    NOLIST
;******* MIKE_RED (E2V CCD44-82) ***********************************************
RLMODE          EQU     $1              ;
RDMODE          EQU     $0              ; $0=normal
HOLD_P          EQU     $1F40           ; P  clock timing ($7D00=400 us)
HOLD_S          EQU     $0006           ; S  clock timing
HOLD_FT         EQU     $1F40           ; FT clock timing ($4E20=250 us)
HOLD_FL         EQU     $1F40           ; FL clock timimg ($3E80=200 us)
HOLD_PL         EQU     $1F40           ; PL settling ($9C40=500us)
HOLD_RG         EQU     $0006           ; RG timing
HOLD_FF         EQU     $0032           ; FF clock timimg
HOLD_IPC        EQU     $0050           ; IPC clock timing
HOLD_SIG        EQU     $0010           ; SIG settling time
HOLD_ADC        EQU     $0010           ; pre-sample settling
INIT_NROWS      EQU     $843            ; $843=2051+64 [4096+6 pix]
INIT_NCOLS      EQU     $459            ; $440=1049+64 [2048+50 pix]
INIT_NFT        EQU     $803            ; $803=[4096+6]/2
INIT_NFLUSH     EQU     $864            ; $864=2048+50+50
INIT_NCH        EQU     $1              ;
INIT_VBIN       EQU     $2              ;
INIT_HBIN       EQU     $2              ;
INIT_VSKIP      EQU     $0              ;
INIT_HSKIP      EQU     $32             ;
INIT_GAIN       EQU     $1              ; 0=LOW 1=HIGH
INIT_USEC       EQU     $C8             ;
INIT_OPCH       EQU     $0              ; BLUE=1 RED=1 newRED=0
INIT_HCLKS      EQU     $1              ; BLUE=2 RED=2 newRED=1
INIT_TCH        EQU     $0              ; 0=TEMP_A, 1=TEMP_B
INIT_PID        EQU     $1              ;
INIT_EXPTIME    EQU     $3E8            ;

;*******************************************************************************
;   INITIALIZE VOLTAGE VALUES (IN X MEMORY)
;*******************************************************************************
            ORG     X:TSET              ;
            DC      $00052B             ; $620 = -100 degC
            DC      $00052B             ; $5EF = -105
            DC      $000000             ; $5BE = -110
            DC      $000000             ; $58C = -115
            DC      $000000             ; $55C = -120
            DC      $000000             ; $52B = -125
            DC      $000000             ; $4FA = -130
            DC      $000000             ; $4C9 = -135

            ORG     X:VBIAS
            DC      (DZ+0040)           ; OFF_A (5 mV/DN)
            DC      (DZ-0000)           ; ADJ_A
            DC      (DZ-1200)           ; OG1_A
            DC      (DZ-1000)           ; 0G2_A
            DC      (DZ+0900)           ; RD_A (10 mV/DN)
            DC      (DZ+0000)           ; SP_A
            DC      (DZ+1500)           ; OFD_A
            DC      (DZ+2100)           ; OD_A
            DC      (DZ+0040)           ; OFF_B (5 mV/DN)
            DC      (DZ-0000)           ; ADJ_B
            DC      (DZ-1200)           ; OG1_B
            DC      (DZ-1000)           ; OG2_B
            DC      (DZ+0900)           ; RD_B (10 mV/DN)
            DC      (DZ+0000)           ; SP_B
            DC      (DZ+1500)           ; OFD_B
            DC      (DZ+2100)           ; OD_B

            ORG     X:VCLK
            DC      (DZ-0000)           ; IPC_A- (5 mV/DN)
            DC      (DZ+1000)           ; IPC_A+
            DC      (DZ-0000)           ; IPC_B-
            DC      (DZ+1000)           ; IPC_B+
            DC      (DZ-1800)           ; RG_A-
            DC      (DZ+0600)           ; RG_A+
            DC      (DZ-1800)           ; RG_B-
            DC      (DZ+0600)           ; RG_B+
            DC      (DZ-1600)           ; SW_A-
            DC      (DZ+0800)           ; SW_A+
            DC      (DZ-1600)           ; SW_B-
            DC      (DZ+0800)           ; SW_B+
            DC      (DZ-1800)           ; TG_A-
            DC      (DZ+0400)           ; TG_A+
            DC      (DZ-1800)           ; TG_B-
            DC      (DZ+0400)           ; TG_B+
            DC      (DZ-1600)           ; S_A-
            DC      (DZ+0800)           ; S_A+
            DC      (DZ-1600)           ; S_B-
            DC      (DZ+0800)           ; S_B+
            DC      (DZ-1800)           ; P1_A-
            DC      (DZ+0400)           ; P1_A+
            DC      (DZ-1800)           ; P2_A-
            DC      (DZ+0400)           ; P2_A+
            DC      (DZ-1800)           ; P3_A-
            DC      (DZ+0400)           ; P3_A+
            DC      (DZ-1800)           ; P1_B-
            DC      (DZ+0400)           ; P1_B+
            DC      (DZ-1800)           ; P2_B-
            DC      (DZ+0400)           ; P2_B+
            DC      (DZ-1800)           ; P3_B-
            DC      (DZ+0400)           ; P3_B+

;*******************************************************************************
;   INITIALIZE SEQ FRAGMENTS (IN X MEMORY)
;*******************************************************************************
;        S2B   _______________  ________________ S1B
;        S3B   ______________ || _______________ S3A
;        SW    _____________ |||| ______________ S2A
;        TG    ____________ |||||| _____________ S1A
;        P1A   ___________ |||||||| ____________ RG
;        P2A   __________ |||||||||| ___________ IPC
;        P3A   _________ |||||||||||| __________ FINT+
;        P1B   ________ |||||||||||||| _________ FINT-
;        P2B   _______ |||||||||||||||| ________ FRST
;        P3B   ______ |||||||||||||||||| _______ CONVST
;                    ||||||||||||||||||||

    IF (RDMODE==$0)                 ; E2V CCD44-82, non-MPP
            ORG X:MPP               ; reset/hold state
            DC  %000001001000011011000011

            ORG X:IPCLKS            ; input clamp
            DC  %000001001000011011010011
            DC  %000001001000011011000011

            ORG X:PCLKS_RD          ; parallel transfer P2-P3-P1-P2
            DC  %000011011000011011000011
            DC  %000010010000011011000011
            DC  %000010110100011011000011
            DC  %000000100100011011000011
            DC  %000001101100011011000011
            DC  %000001001000011011000011

            ORG X:PCLKS_FL          ; parallel flush P2-P3-P1-P2
            DC  %000011011000011011000011
            DC  %000010010000011011000011
            DC  %000010110100011011000011
            DC  %000000100100011011000011
            DC  %000001101100011011000011
            DC  %000001001000011011000011

            ORG X:INT_L             ; reset and integrate left
            DC  %000001001000010010100011   ; RG ON  FRST ON
            DC  %000001001000010010000011   ; RG OFF
            DC  %000001001001010010000001   ; FRST OFF SW ON
            DC  %000001001001010010001001   ; FINT+ ON
            DC  %000001001001010010000001   ; FINT+ OFF
            DC  %000001001000010010000001   ; SW OFF
            DC  %000001001000010010000101   ; FINT- ON
            DC  %000001001000010010000001   ; FINT- OFF
            DC  %000001001000010010000000   ; /CONVST ON
            DC  %000001001000010010000001   ; /CONVST OFF
            DC  %000001001000010010000011   ; FRST ON  [note: IPC !!]

            ORG X:INT_R             ; reset and integrate right
            DC  %000001001000001001100011   ; RG ON  FRST ON
            DC  %000001001000001001000011   ; RG OFF
            DC  %000001001001001001000001   ; FRST OFF SW ON
            DC  %000001001001001001001001   ; FINT+ ON
            DC  %000001001001001001000001   ; FINT+ OFF
            DC  %000001001000001001000001   ; SW OFF
            DC  %000001001000001001000101   ; FINT- ON
            DC  %000001001000001001000001   ; FINT- OFF
            DC  %000001001000001001000000   ; /CONVST ON
            DC  %000001001000001001000001   ; /CONVST OFF
            DC  %000001001000001001000011   ; FRST ON

            ORG X:SCLKS_L           ; serial shift (left=A) S2-S1-S3-S2
            DC  %000001001001011011010011   ; note IPC!!
            DC  %000001001001001001010011
            DC  %000001001001101101010011
            DC  %000001001001100100010011
            DC  %000001001001110110010011
            DC  %000001001001010010010011

            ORG X:SCLKS_R           ; serial shift (right=B) S1-S2-S3-S1
            DC  %000001001001011011010011
            DC  %000001001001010010010011
            DC  %000001001001110110010011
            DC  %000001001001100100010011
            DC  %000001001001101101010011
            DC  %000001001001001001010011

            ORG X:SCLKS_FLL         ; serial flush (left=A) S2-S1-S3-S2
            DC  %000001001001011011100011
            DC  %000001001001001001100011
            DC  %000001001001101101100011
            DC  %000001001001100100100011
            DC  %000001001001110110100011
            DC  %000001001000010010000011

            ORG X:SCLKS_FLR         ; serial flush (right=B) S1-S2-S3-S1
            DC  %000001001001011011100011
            DC  %000001001001010010100011
            DC  %000001001001110110100011
            DC  %000001001001100100100011
            DC  %000001001001101101100011
            DC  %000001001000001001000011

            ORG X:PCLKS_FWD          ; parallel shuffle P2-P3-P1-P2
            DC  %000011001100011011000011   ; P2-P1-P3-P2
            DC  %000010000100011011000011
            DC  %000010110100011011000011
            DC  %000000110000011011000011
            DC  %000001111000011011000011
            DC  %000001001000011011000011

            ORG X:PCLKS_REV          ; parallel shuffle P2-P1-P3-P2
            DC  %000001111000011011000011   ; P2-P3-P1-P2
            DC  %000000110000011011000011
            DC  %000010110100011011000011
            DC  %000010000100011011000011
            DC  %000011001100011011000011
            DC  %000001001000011011000011
    ENDIF


