;*******************************************************************************
;   DEV_SITE.ASM -- DEVICE SPECIFIC OPTIONS, CLOCKING SEQUENCES & VOLTAGES
;*******************************************************************************
    NOLIST
;******* SITe (ST-002A) ********************************************************
RLMODE          EQU     $1              ;
RDMODE          EQU     $0              ; $0=normal
HOLD_P          EQU     $4E20           ; P  clock timing ($7D00=400 us)
HOLD_S          EQU     $0006           ; S  clock timing
HOLD_FT         EQU     $4E20           ; FT clock timing ($4E20=250 us)
HOLD_FL         EQU     $4E20           ; FL clock timimg ($3E80=200 us)
HOLD_PL         EQU     $7D00           ; PL settling ($9C40=500us)
HOLD_RG         EQU     $0006           ; RG timing (was $6)
HOLD_FF         EQU     $0032           ; FF clock timimg
HOLD_IPC        EQU     $0000           ; IPC clock timing
HOLD_SIG        EQU     $0020           ; SIG settling time $30
HOLD_ADC        EQU     $0010           ; pre-sample settling $8
INIT_NROWS      EQU     $840            ; $840=2048+64
INIT_NCOLS      EQU     $440            ; $440=1024+64
INIT_NFT        EQU     $800            ; $800=2048
INIT_NFLUSH     EQU     $830            ; $840=2048+24+24
INIT_NCH        EQU     $1              ;
INIT_VBIN       EQU     $2              ;
INIT_HBIN       EQU     $2              ;
INIT_VSKIP      EQU     $0              ;
INIT_HSKIP      EQU     $18             ;
INIT_GAIN       EQU     $0              ; 0=LOW 1=HIGH
INIT_USEC       EQU     $C8             ;
INIT_OPCH       EQU     $0              ; site_BLUE=0 e2vRED=0
INIT_HCLKS      EQU     $1              ; site_BLUE=1 e2vRED=1
INIT_TCH        EQU     $0              ; 0=TEMP_A, 1=TEMP_B
INIT_PID        EQU     $1              ;
INIT_EXPTIME    EQU     $3E8            ;

;*******************************************************************************
;   INITIALIZE VOLTAGE VALUES (IN X MEMORY)
;*******************************************************************************
            ORG     X:TSET              ;
            DC      $00052B             ; $620 = -100 degC
            DC      $00052B             ; $5EF = -105
            DC      $000000             ; $5BE = -110
            DC      $000000             ; $58C = -115
            DC      $000000             ; $55C = -120
            DC      $000000             ; $52B = -125
            DC      $000000             ; $4FA = -130
            DC      $000000             ; $4C9 = -135

            ORG     X:VBIAS
            DC      (DZ-0000)           ; OFF_A (5 mV/DN)
            DC      (DZ-0148)           ; ADJ_A
            DC      (DZ-0800)           ; OG_A
            DC      (DZ+0000)           ; ABG_A
            DC      (DZ+1350)           ; RD_A (10 mV/DN)
            DC      (DZ+0000)           ; SP_A
            DC      (DZ+2000)           ; OFD_A
            DC      (DZ+2400)           ; OD_A
            DC      (DZ-0000)           ; OFF_B (5 mV/DN)
            DC      (DZ-0148)           ; ADJ_B
            DC      (DZ-0800)           ; OG_B
            DC      (DZ+0000)           ; ABG_B
            DC      (DZ+1350)           ; RD_B (10 mV/DN)
            DC      (DZ+0000)           ; SP_B
            DC      (DZ+2000)           ; OFD_B
            DC      (DZ+2500)           ; OD_B

            ORG     X:VCLK
            DC      (DZ-0000)           ; IPC_A- (5 mV/DN)
            DC      (DZ+1000)           ; IPC_A+
            DC      (DZ-0000)           ; IPC_B-
            DC      (DZ+1000)           ; IPC_B+
            DC      (DZ-0000)           ; RG_A-
            DC      (DZ+2400)           ; RG_A+
            DC      (DZ-0000)           ; RG_B-
            DC      (DZ+2400)           ; RG_B+
            DC      (DZ-1200)           ; SW_A-
            DC      (DZ+1000)           ; SW_A+
            DC      (DZ-1200)           ; SW_B-
            DC      (DZ+1000)           ; SW_B+
            DC      (DZ-0000)           ; TG_A-
            DC      (DZ+0000)           ; TG_A+
            DC      (DZ-0000)           ; TG_B-
            DC      (DZ+0000)           ; TG_B+
            DC      (DZ-1200)           ; S_A-
            DC      (DZ+1000)           ; S_A+
            DC      (DZ-1200)           ; S_B-
            DC      (DZ+1000)           ; S_B+
            DC      (DZ-1800)           ; P1_A-
            DC      (DZ+0500)           ; P1_A+
            DC      (DZ-1800)           ; P2_A-
            DC      (DZ+0500)           ; P2_A+
            DC      (DZ-1300)           ; P3_A-
            DC      (DZ+1600)           ; P3_A+
            DC      (DZ-1800)           ; P1_B-
            DC      (DZ+0500)           ; P1_B+
            DC      (DZ-1800)           ; P2_B-
            DC      (DZ+0500)           ; P2_B+
            DC      (DZ-1300)           ; P3_B-
            DC      (DZ+1600)           ; P3_B+

;*******************************************************************************
;   INITIALIZE SEQ FRAGMENTS (IN X MEMORY)
;*******************************************************************************
;        S2B   _______________  ________________ S1B
;        S3B   ______________ || _______________ S3A
;        SW    _____________ |||| ______________ S2A
;        TG    ____________ |||||| _____________ S1A
;        P1A   ___________ |||||||| ____________ RG
;        P2A   __________ |||||||||| ___________ IPC
;        P3A   _________ |||||||||||| __________ FINT+
;        P1B   ________ |||||||||||||| _________ FINT-
;        P2B   _______ |||||||||||||||| ________ FRST
;        P3B   ______ |||||||||||||||||| _______ CONVST
;                    ||||||||||||||||||||

    IF (RDMODE==$0)                 ; SITe ST-002A  MPP=PHASE_3
            ORG X:MPP               ; reset/hold state
            DC  %000000000000001001000011

            ORG X:IPCLKS            ; input clamp
            DC  %000000000000001001010011
            DC  %000000000000001001000011

            ORG X:PCLKS_RD          ; parallel transfer P2-P3-P1
            DC  %000001001000001001000011
            DC  %000011011000001001000011
            DC  %000010010000001001000011
            DC  %000010110100001001000011
            DC  %000000100100001001000011
            DC  %000000000000001001000011

            ORG X:PCLKS_FL          ; parallel flush P2-P3-P1
            DC  %000001001000001001000011
            DC  %000011011000001001000011
            DC  %000010010000001001000011
            DC  %000010110100001001000011
            DC  %000000100100001001000011
            DC  %000000000000001001000011

            ORG X:INT_L            ; reset and integrate left
            DC  %000000000000001001100011   ; RG ON  FRST ON
            DC  %000000000000001001000011   ; RG OFF
            DC  %000000000001001001000001   ; FRST OFF SW ON
            DC  %000000000001001001001001   ; FINT+ ON
            DC  %000000000001001001000001   ; FINT+ OFF
            DC  %000000000000001001000001   ; SW OFF
            DC  %000000000000001001000101   ; FINT- ON
            DC  %000000000000001001000001   ; FINT- OFF
            DC  %000000000000001001000000   ; /CONVST ON
            DC  %000000000000001001000001   ; /CONVST OFF
            DC  %000000000000001001000011   ; FRST ON

            ORG X:INT_R            ; reset and integrate right
            DC  %000000000000001001100011   ; RG ON  FRST ON
            DC  %000000000000001001000011   ; RG OFF
            DC  %000000000001001001000001   ; FRST OFF SW ON
            DC  %000000000001001001001001   ; FINT+ ON
            DC  %000000000001001001000001   ; FINT+ OFF
            DC  %000000000000001001000001   ; SW OFF
            DC  %000000000000001001000101   ; FINT- ON
            DC  %000000000000001001000001   ; FINT- OFF
            DC  %000000000000001001000000   ; /CONVST ON
            DC  %000000000000001001000001   ; /CONVST OFF
            DC  %000000000000001001000011   ; FRST ON

            ORG X:SCLKS_L           ; serial shift (left=A) S1-S3-S2-S1
            DC  %000000000001101101000011
            DC  %000000000001100100000011
            DC  %000000000001110110000011
            DC  %000000000001010010000011
            DC  %000000000001011011000011
            DC  %000000000001001001000011

            ORG X:SCLKS_R           ; serial shift (right=B) S1-S2-S3-S1
            DC  %000000000001011011000011
            DC  %000000000001010010000011
            DC  %000000000001110110000011
            DC  %000000000001100100000011
            DC  %000000000001101101000011
            DC  %000000000001001001000011

            ORG X:SCLKS_FLL         ; serial flush (left=A) S1-S3-S2-S1
            DC  %000000000001101101100011
            DC  %000000000001100100100011
            DC  %000000000001110110100011
            DC  %000000000001010010100011
            DC  %000000000001011011100011
            DC  %000000000001001001000011

            ORG X:SCLKS_FLR         ; serial flush (right=B) S1-S2-S3-S1
            DC  %000000000001011011100011
            DC  %000000000001010010100011
            DC  %000000000001110110100011
            DC  %000000000001100100100011
            DC  %000000000001101101100011
            DC  %000000000001001001000011

;           ORG X:SCLKS_FF          ; serial flush (fast) DG
;           DC  %000000000001111111100011
;           DC  %000000000001111111100011
;           DC  %000000000001111111100011
;           DC  %000000000001111111100011
;           DC  %000000000001111111100011   ; dummy code
;           DC  %000000000000001001000011   ; dummy code

            ORG X:PCLKS_FWD          ; parallel shuffle  P2-P3-P1
            DC  %000001000100001001000011               ; P1-P3-P2
            DC  %000011010100001001000011
            DC  %000010010000001001000011
            DC  %000010111000001001000011
            DC  %000000101000001001000011
            DC  %000000000000001001000011

            ORG X:PCLKS_REV          ; parallel shuffle   P1-P3-P2
            DC  %000000101000001001000011               ; P2-P3-P1
            DC  %000010111000001001000011
            DC  %000010010000001001000011
            DC  %000011010100001001000011
            DC  %000001000100001001000011
            DC  %000000000000001001000011
    ENDIF
