;*******************************************************************************
;   DEV_WINGCHIP.ASM -- DEVICE SPECIFIC OPTIONS, CLOCKING SEQUENCES & VOLTAGES
;*******************************************************************************
    NOLIST
;******* WING_CHIP (STA 2560x512) **********************************************
RLMODE          EQU     $2              ;
RDMODE          EQU     $0              ; $0=normal
HOLD_P          EQU     $4E20           ; P  clock timing ($4E20=250 us)
HOLD_S          EQU     $0006           ; S  clock timing
HOLD_FT         EQU     $4E20           ; FT clock timing
HOLD_FL         EQU     $4E20           ; FL clock timimg
HOLD_PL         EQU     $1F40           ; PL settling ($1F40=100us)
HOLD_RG         EQU     $0006           ; RG timing
HOLD_FF         EQU     $0032           ; FF clock timimg
HOLD_IPC        EQU     $0000           ; IPC clock timing
HOLD_SIG        EQU     $0010           ; SIG settling time
HOLD_ADC        EQU     $0010           ; pre-sample settling
INIT_NROWS      EQU     $140            ; $240=512+64  $140=256+64
INIT_NCOLS      EQU     $540            ; $A40=2560+64 $540=1280+64
INIT_NFT        EQU     $200            ; $200=512
INIT_NFLUSH     EQU     $A00            ; $A00=2560
INIT_NCH        EQU     $2              ;
INIT_VBIN       EQU     $2              ;
INIT_HBIN       EQU     $2              ;
INIT_VSKIP      EQU     $0              ;
INIT_HSKIP      EQU     $0              ;
INIT_GAIN       EQU     $0              ; 0=LOW 1=HIGH
INIT_USEC       EQU     $C8             ;
INIT_OPCH       EQU     $0              ;
INIT_HCLKS      EQU     $1              ;
INIT_TCH        EQU     $0              ; 0=TEMP_A, 1=TEMP_B
INIT_PID        EQU     $0              ;
INIT_EXPTIME    EQU     $3E8            ;

;*******************************************************************************
;   INITIALIZE VOLTAGE VALUES (IN X MEMORY)
;*******************************************************************************
            ORG     X:TSET              ;
            DC      $000000             ; $682 = - 90 degC
            DC      $000000             ; $651 = - 95
            DC      $000000             ; $620 = -100
            DC      $000000             ; $5EF = -105
            DC      $000000             ; $5BE = -110
            DC      $000000             ; $58C = -115
            DC      $000000             ; $55C = -120
            DC      $000000             ; $52B = -125

            ORG     X:VBIAS
            DC      (DZ+1000)           ; OFF_A (5 mV/DN)
            DC      (DZ-0000)           ; ADJ_A
            DC      (DZ-0000)           ; OG_A
            DC      (DZ+0000)           ; ABG_A
            DC      (DZ+1600)           ; RD_A (10 mV/DN)
            DC      (DZ+0000)           ; SP_A
            DC      (DZ+0000)           ; OFD_A
            DC      (DZ+2400)           ; OD_A
            DC      (DZ+1000)           ; OFF_B (5 mV/DN)
            DC      (DZ-0200)           ; ADJ_B
            DC      (DZ-0050)           ; OG_B
            DC      (DZ+0000)           ; ABG_B
            DC      (DZ+1600)           ; RD_B (10 mV/DN)
            DC      (DZ+0000)           ; SP_B
            DC      (DZ+0000)           ; OFD_B
            DC      (DZ+2400)           ; OD_B

            ORG     X:VCLK
            DC      (DZ-0000)           ; IPC_A- (5 mV/DN)
            DC      (DZ+1000)           ; IPC_A+
            DC      (DZ-0000)           ; IPC_B-
            DC      (DZ+1000)           ; IPC_B+
            DC      (DZ-1000)           ; RG_A-
            DC      (DZ+1000)           ; RG_A+
            DC      (DZ-1000)           ; RG_B-
            DC      (DZ+1000)           ; RG_B+
            DC      (DZ-1000)           ; SW_A-
            DC      (DZ+1000)           ; SW_A+
            DC      (DZ-1000)           ; SW_B-
            DC      (DZ+1000)           ; SW_B+
            DC      (DZ-0000)           ; TG_A-
            DC      (DZ+0000)           ; TG_A+
            DC      (DZ-0000)           ; TG_B-
            DC      (DZ+0000)           ; TG_B+
            DC      (DZ-1000)           ; S_A-
            DC      (DZ+1000)           ; S_A+
            DC      (DZ-1000)           ; S_B-
            DC      (DZ+1000)           ; S_B+
            DC      (DZ-1600)           ; P1_A-
            DC      (DZ+0400)           ; P1_A+
            DC      (DZ-1600)           ; P2_A-
            DC      (DZ+0400)           ; P2_A+
            DC      (DZ-1600)           ; P3_A-
            DC      (DZ+0400)           ; P3_A+
            DC      (DZ-1600)           ; P1_B-
            DC      (DZ+0400)           ; P1_B+
            DC      (DZ-1600)           ; P2_B-
            DC      (DZ+0400)           ; P2_B+
            DC      (DZ-1600)           ; P3_B-
            DC      (DZ+0400)           ; P3_B+

;*******************************************************************************
;   INITIALIZE SEQ FRAGMENTS (IN X MEMORY)
;*******************************************************************************
;        S2B   _______________  ________________ S1B
;        S3B   ______________ || _______________ S3A
;        SW    _____________ |||| ______________ S2A
;        TG    ____________ |||||| _____________ S1A
;        P1A   ___________ |||||||| ____________ RG
;        P2A   __________ |||||||||| ___________ IPC
;        P3A   _________ |||||||||||| __________ FINT+
;        P1B   ________ |||||||||||||| _________ FINT-
;        P2B   _______ |||||||||||||||| ________ FRST
;        P3B   ______ |||||||||||||||||| _______ CONVST
;                    ||||||||||||||||||||

    IF (RDMODE==$0)                 ; wing-chips, non-MPP
            ORG X:MPP               ; reset/hold state
            DC  %000001001000110110000011

            ORG X:IPCLKS            ; input clamp
            DC  %000001001000110110010011
            DC  %000001001000110110000011

            ORG X:PCLKS_RD          ; parallel transfer P2-P3-P1-P2
            DC  %000011011000110110000011
            DC  %000010010000110110000011
            DC  %000010110100110110000011
            DC  %000000100100110110000011
            DC  %000001101100110110000011
            DC  %000001001000110110000011

            ORG X:PCLKS_FL          ; parallel flush P2-P3-P1-P2
            DC  %000011011000110110000011
            DC  %000010010000110110000011
            DC  %000010110100110110000011
            DC  %000000100100110110000011
            DC  %000001101100110110000011
            DC  %000001001000110110000011

            ORG X:INT_L             ; reset and integrate left
            DC  %000001001000010010100011   ; RG ON  FRST ON
            DC  %000001001000010010000011   ; RG OFF
            DC  %000001001001010010000001   ; FRST OFF SW ON
            DC  %000001001001010010001001   ; FINT+ ON
            DC  %000001001001010010000001   ; FINT+ OFF
            DC  %000001001000010010000001   ; SW OFF
            DC  %000001001000010010000101   ; FINT- ON
            DC  %000001001000010010000001   ; FINT- OFF
            DC  %000001001000010010000000   ; /CONVST ON
            DC  %000001001000010010000001   ; /CONVST OFF
            DC  %000001001000010010000011   ; FRST ON

            ORG X:INT_R             ; reset and integrate right
            DC  %000001001000010010100011   ; RG ON  FRST ON
            DC  %000001001000010010000011   ; RG OFF
            DC  %000001001001010010000001   ; FRST OFF SW ON
            DC  %000001001001010010001001   ; FINT+ ON
            DC  %000001001001010010000001   ; FINT+ OFF
            DC  %000001001000010010000001   ; SW OFF
            DC  %000001001000010010000101   ; FINT- ON
            DC  %000001001000010010000001   ; FINT- OFF
            DC  %000001001000010010000000   ; /CONVST ON
            DC  %000001001000010010000001   ; /CONVST OFF
            DC  %000001001000010010000011   ; FRST ON

            ORG X:SCLKS_L           ; serial shift (left=A) S2-S1-S3-S2
            DC  %000001001001011011000011
            DC  %000001001001001001000011
            DC  %000001001001101101000011
            DC  %000001001001100100000011
            DC  %000001001001110110000011
            DC  %000001001001010010000011

            ORG X:SCLKS_R           ; serial shift (right=B) S2-S3-S1-S2
            DC  %000001001001110110000011
            DC  %000001001001100100000011
            DC  %000001001001101101000011
            DC  %000001001001001001000011
            DC  %000001001001011011000011
            DC  %000001001001010010000011

            ORG X:SCLKS_FLL         ; serial flush (left=A) S2-S1-S3-S2
            DC  %000001001001011011100011
            DC  %000001001001001001100011
            DC  %000001001001101101100011
            DC  %000001001001100100100011
            DC  %000001001001110110100011
            DC  %000001001000010010000011

            ORG X:SCLKS_FLR         ; serial flush (right=B) S2-S3-S1-S2
            DC  %000001001001110110100011
            DC  %000001001001100100100011
            DC  %000001001001101101100011
            DC  %000001001001001001100011
            DC  %000001001001011011100011
            DC  %000001001000010010000011
    ENDIF


