;*****************************************************************************
;   ROUTINE TO SET D/A VOLTAGES AND EXERCISE CCD DRIVER OP-AMPS
;*****************************************************************************
    PAGE    110,60,1,1
    TABS    4
;*****************************************************************************
;   DEFINITIONS & POINTERS
;*****************************************************************************
START       EQU     $000100             ; program start location
DZ          EQU     $001000             ; DAC zero volt offset

SEQREG      EQU     $FFFF88             ; external CCD clock register
HR          EQU     $FFFF80             ; Heater resistors
CLK_AB      EQU     $FFFD00             ; A+B clocks
CLK_CD      EQU     $FFFD40             ; C+D
CLK_EF      EQU     $FFFD80             ; E+F
CLK_GH      EQU     $FFFDC0             ; G+H
SIG_AB      EQU     $FFFC10             ; A+B bias
SIG_CD      EQU     $FFFC30             ; C+D
SIG_EF      EQU     $FFFC50             ; E+F
SIG_GH      EQU     $FFFC70             ; G+H

AAR3        EQU     $FFFFF6             ; Address attribute register 3
BCR         EQU     $FFFFFB             ; Bus control register (wait states)

WAIT1       EQU     $073FE1             ; 1 PERIPH 1 SRAM 31 EPROM
WAIT5       EQU     $07BFE1             ; 5 PERIPH 1 SRAM 31 EPROM

;*****************************************************************************
;   MAIN PROGRAM
;*****************************************************************************
            ORG     P:$0
            JMP     START

            ORG     P:START
SET_MODE    MOVEP   #$FFFC21,X:AAR3             ; PERIPH $FFF000--$FFFFFF

DAC_SETUP   MOVEP   #WAIT5,X:BCR                ; add wait states
            MOVE    #(DZ+400),X0                ;
            MOVE    #(DZ-400),Y0                ;
            MOVE    #HR,R0                      ; set heater voltages
            DO      #$8,END_VHEAT               ;
            MOVE    X0,Y:(R0)+                  ;
END_VHEAT   NOP

            MOVE    #(DZ+400),X0                ;
            MOVE    #SIG_AB,R0                  ; set bias voltages
            DO      #$10,END_B0                 ;
            MOVE    X0,Y:(R0)+                  ;
END_B0      MOVE    #(DZ+500),X0                ;
            MOVE    #SIG_CD,R0                  ; set bias voltages
            DO      #$10,END_B1                 ;
            MOVE    X0,Y:(R0)+                  ;
END_B1      MOVE    #(DZ+600),X0                ;
            MOVE    #SIG_EF,R0                  ; set bias voltages
            DO      #$10,END_B2                 ;
            MOVE    X0,Y:(R0)+                  ;
END_B2      MOVE    #(DZ+700),X0                ;
            MOVE    #SIG_GH,R0                  ; set bias voltages
            DO      #$10,END_B3                 ;
            MOVE    X0,Y:(R0)+                  ;
END_B3      NOP                                 ;

            MOVE    #(DZ+400),X0                ;
            MOVE    #(DZ-400),Y0                ;
            MOVE    #CLK_AB,R0                  ; set clock voltages
            DO      #$10,END_V0                 ;
            MOVE    Y0,Y:(R0)+                  ;
            MOVE    X0,Y:(R0)+                  ;
END_V0      MOVE    #(DZ+500),X0                ;
            MOVE    #(DZ-500),Y0                ;
            MOVE    #CLK_CD,R0                  ;
            DO      #$10,END_V1                 ;
            MOVE    Y0,Y:(R0)+                  ;
            MOVE    X0,Y:(R0)+                  ;
END_V1      MOVE    #(DZ+600),X0                ;
            MOVE    #(DZ-600),Y0                ;
            MOVE    #CLK_EF,R0                  ;
            DO      #$10,END_V2                 ;
            MOVE    Y0,Y:(R0)+                  ;
            MOVE    X0,Y:(R0)+                  ;
END_V2      MOVE    #(DZ+700),X0                ;
            MOVE    #(DZ-700),Y0                ;
            MOVE    #CLK_GH,R0                  ;
            DO      #$10,END_V3                 ;
            MOVE    Y0,Y:(R0)+                  ;
            MOVE    X0,Y:(R0)+                  ;
END_V3      MOVEP   #WAIT1,X:BCR                ;re-set wait states

            MOVE    #>$4000,X0                  ;

MAIN        MOVEP   #>$FFFFF3,Y:<<SEQREG        ; exercise the clocks
            REP     X0                          ; small delay
            NOP                                 ;
            MOVEP   #>$000003,Y:<<SEQREG        ;
            REP     X0                          ; small delay
            NOP                                 ;
            JMP     MAIN                        ;

            END

