;*****************************************************************************
;   SERIAL.ASM -- SILICON SERIAL NUMBER TEST PROGRAM
;*****************************************************************************
    PAGE    110,60,1,1
    TABS    4
;*****************************************************************************
;   DEFINITIONS & POINTERS
;*****************************************************************************
START       EQU     $000100                 ; program start location
SERIAL      EQU     $2                      ; serial number location

TXD         EQU     $FFFFBC                 ; ESSI0 Transmit Data Register 0
RXD         EQU     $FFFFB8                 ; ESSI0 Receive Data Register
SSISR       EQU     $FFFFB7                 ; ESSI0 Status Register
CRB         EQU     $FFFFB6                 ; ESSI0 Control Register B
CRA         EQU     $FFFFB5                 ; ESSI0 Control Register A
PCRC        EQU     $FFFFBF                 ; Port C (ESSI_0) control register
PCTL        EQU     $FFFFFD                 ; PLL control register
TXREG       EQU     $FFFF89                 ; external TX register
RXREG       EQU     $FFFF8A                 ; external RX register

PDRB        EQU     $FFFFC9                 ; Port B (HOST) GPIO data
PRRB        EQU     $FFFFC8                 ; Port B (HOST) GPIO direction
PCRB        EQU     $FFFFC4                 ; Port B (HOST) control register

PCRE        EQU     $FFFF9F                 ; Port E (SCI) control register
PRRE        EQU     $FFFF9E                 ; Port E (SCI) data direction
PDRE        EQU     $FFFF9D                 ; Port E (SCI) data

;*****************************************************************************
;   INITIALIZE INTERRUPT VECTORS
;*****************************************************************************
            ORG     P:$0
            JMP     START

;*****************************************************************************
;   MAIN PROGRAM
;*****************************************************************************
            ORG     P:START
PLL_SETUP   MOVEP   #$078007,X:PCTL         ; set PLL parameters

PORTB_SETUP MOVEP   #>$1,X:PCRB             ; set PB[15..0] GPIO
            MOVEP   #>$FE,X:PRRB            ; TM GAIN .. OPT3V

SSI_SETUP   MOVEP   #>$032070,X:CRB         ; async, LSB, disable TE RE
            MOVEP   #>$140803,X:CRA         ; 10 Mbps, 16 bit word
            MOVEP   #>$3F,X:PCRC            ; enable ESSI

PORTE_SETUP MOVEP   #$0,X:PCRE              ; disable SCI
            MOVEP   #>$2,X:PRRE             ;
            MOVEP   #$2,X:PDRE              ; close shutter

            MOVE    #>$A,X0                 ;
            MOVE    X0,X:(SERIAL+0)         ;
            MOVE    #>$B,X0                 ;
            MOVE    X0,X:(SERIAL+1)         ;
            MOVE    #>$C,X0                 ;
            MOVE    X0,X:(SERIAL+2)         ;
            MOVE    #>$D,X0                 ;
            MOVE    X0,X:(SERIAL+3)         ;

MAIN        JSR     READ16                  ;
            REP     #$28                    ;
            NOP                             ;

            JSR     W1_READ                 ;

            MOVE    X:(SERIAL+0),A          ;
            JSR     WRITE16                 ;
            MOVE    X:(SERIAL+1),A          ;
            JSR     WRITE16                 ;
            MOVE    X:(SERIAL+2),A          ;
            JSR     WRITE16                 ;
            MOVE    X:(SERIAL+3),A          ;
            JSR     WRITE16                 ;
            JMP     MAIN                    ;

;*******************************************************************************
;   READ AND WRITE 16-BIT DATA
;*******************************************************************************
WRITE16     JCLR    #$8,X:PDRB,WR16B        ;[ ] check RS485 or fiber
            MOVE    A1,Y:TXREG              ;[ ] write bits 15..0
WR16B       JSET    #$8,X:PDRB,ENDWR16      ;[ ]
            JCLR    #6,X:SSISR,*            ;[4] wait for TDE
            MOVE    A1,X:TXD                ;[ ]
ENDWR16     RTS                             ;[3]

READ16      JCLR    #$8,X:PDRB,RD16B        ;[ ] check RS485 or fiber
            JCLR    #$A,X:PDRB,*            ;[ ] wait for data in RXREG
            MOVE    Y:RXREG,A               ;[ ] write bits 15..0
RD16B       JSET    #$8,X:PDRB,ENDRD16      ;[ ] check RS485 or fiber
            JCLR    #7,X:SSISR,*            ;[4] wait for RDRF to go high
            MOVE    X:RXD,A                 ;[2] read from ESSI
            NOP                             ;[ ]
ENDRD16     RTS                             ;[3] 16-bit word in A1

;*******************************************************************************
;   ONE-WIRE ROUTINE -- DS2401 USING PB0 
;*******************************************************************************
W1_READ     MOVE    #SERIAL,R3              ; DS2401 1-wire read routine

W1_RESET    BCLR    #$0,X:PDRB              ; set PB0=0
            BSET    #$0,X:PRRB              ; output PE2=0            
            MOVE    #>$9600,X0              ; wait $9600=480us
            REP     X0                      ;
            NOP                             ;
            BCLR    #$0,X:PRRB              ; set PB0 input
            MOVE    #>$15E0,X0              ; wait $15E0=70us
            REP     X0                      ;
            NOP                             ;
            MOVEP   X:PDRB,X1               ; sample PB0, get ACK
            MOVE    #>$8020,X0              ; wait $8020=410us
            REP     X0                      ;
            NOP                             ; 

W1_WR       MOVE    #$33,A1                 ; data to xmit
            DO      #$8,END_WR              ; set up 8 clocks
WCLKS       BCLR    #$0,X:PDRB              ; set PB0=0
            BSET    #$0,X:PRRB              ; output PB0=0
            MOVE    #>$1E0,X0               ; wait $1E0=6us
            REP     X0                      ;
            NOP                             ;
WD0         JSET    #$0,A1,WD1              ;
            BSET    #$0,X:PRRB              ; output PB0=0
            MOVE    #>$10E0,X0              ; wait $10E0=54us
            REP     X0                      ;
            NOP                             ; 
            BCLR    #$0,X:PRRB              ; PB0 hi-Z
WD1         JCLR    #$0,A1,WD2              ;
            BCLR    #$0,X:PRRB              ; PB0=hi-Z
            MOVE    #>$10E0,X0              ; wait $10E0=54us
            REP     X0                      ;
            NOP                             ; 
WD2         BCLR    #$0,X:PRRB              ; PB0 hi-Z
            MOVE    #>$320,X0               ; wait $320=10us
            REP     X0                      ;
            NOP                             ;
            ASR     A                       ;
END_WR      NOP                             ;

W1_RD       DO      #$4,END_RD0             ;
RD0         CLR     A                       ;
RD1         DO      #$10,END_RD1            ; set up 16 clocks
            BCLR    #$0,X:PDRB              ; set PB0=0
            BSET    #$0,X:PRRB              ; output PB0=0
            MOVE    #>$1E0,X0               ; wait $1E0=6us
            REP     X0                      ;
            NOP                             ; 
            BCLR    #$0,X:PRRB              ; PB0 hi-Z
            MOVE    #>$2D0,X0               ; wait $2D0=9us
            REP     X0                      ;
            NOP                             ;
            MOVEP   X:PDRB,X1               ; sample PB0
RB0         JSET    #$0,X1,RB1              ;
            BCLR    #$0,A1                  ; bit=0
RB1         JCLR    #$0,X1,RB2              ;
            BSET    #$0,A1                  ; bit=1
RB2         NOP                             ;
            ASR     A                       ;
            MOVE    #>$1130,X0              ; wait $1130=55us
            REP     X0                      ;
            NOP                             ;
END_RD1     NOP                             ;
            ASR     #$8,A,A                 ;
            NOP
            MOVE    A0,X:(R3)+              ;
END_RD0     NOP                             ;

            BCLR    #$0,X:PRRB              ; set PB0 hi-Z
            RTS                             ;

