;*****************************************************************************
;   SLINK.ASM -- SERIAL LINK TEST PROGRAM
;*****************************************************************************
    PAGE    110,60,1,1
    TABS    4
;*****************************************************************************
;   DEFINITIONS & POINTERS
;*****************************************************************************
START       EQU     $000100                 ; program start location
TXD         EQU     $FFFFBC                 ; ESSI0 Transmit Data Register 0
RXD         EQU     $FFFFB8                 ; ESSI0 Receive Data Register
SSISR       EQU     $FFFFB7                 ; ESSI0 Status Register
CRB         EQU     $FFFFB6                 ; ESSI0 Control Register B
CRA         EQU     $FFFFB5                 ; ESSI0 Control Register A
PCRC        EQU     $FFFFBF                 ; Port C (ESSI_0) control register
PCTL        EQU     $FFFFFD                 ; PLL control register
TXREG       EQU     $FFFF89                 ; external TX register
RXREG       EQU     $FFFF8A                 ; external RX register

PDRB        EQU     $FFFFC9                 ; Port B (HOST) GPIO data
PRRB        EQU     $FFFFC8                 ; Port B (HOST) GPIO direction
PCRB        EQU     $FFFFC4                 ; Port B (HOST) control register

;*****************************************************************************
;   INITIALIZE INTERRUPT VECTORS
;*****************************************************************************
            ORG     P:$0
            JMP     START

;*****************************************************************************
;   MAIN PROGRAM
;*****************************************************************************
            ORG     P:START
PLL_SETUP   MOVEP   #$078007,X:PCTL         ; set PLL parameters

PORTB_SETUP MOVEP   #>$1,X:PCRB             ; set PB[15..0] GPIO
            MOVEP   #>$FF,X:PRRB            ; TM GAIN .. OPT3V

SSI_SETUP   MOVEP   #>$032070,X:CRB         ; async, LSB, disable TE RE
            MOVEP   #>$140803,X:CRA         ; 10 Mbps, 16 bit word
            MOVEP   #>$3F,X:PCRC            ; enable ESSI

MAIN        JSR     READ16
            REP     #$28
            NOP

            JSR     WRITE16
            REP     #$28
            NOP
            JMP     MAIN

;*******************************************************************************
;   READ AND WRITE 16-BIT DATA
;*******************************************************************************
WRITE16     JCLR    #$8,X:PDRB,WR16B        ;[ ] check RS485 or fiber
            MOVE    A1,Y:TXREG              ;[ ] write bits 15..0
WR16B       JSET    #$8,X:PDRB,ENDWR16      ;[ ]
            JCLR    #6,X:SSISR,*            ;[4] wait for TDE
            MOVE    A1,X:TXD                ;[ ]
ENDWR16     RTS                             ;[3]

READ16      JCLR    #$8,X:PDRB,RD16B        ;[ ] check RS485 or fiber
            JCLR    #$A,X:PDRB,*            ;[ ] wait for data in RXREG
            MOVE    Y:RXREG,A               ;[ ] write bits 15..0
RD16B       JSET    #$8,X:PDRB,ENDRD16      ;[ ] check RS485 or fiber
            JCLR    #7,X:SSISR,*            ;[4] wait for RDRF to go high
            MOVE    X:RXD,A                 ;[2] read from ESSI
            NOP                             ;[ ]
ENDRD16     RTS                             ;[3] 16-bit word in A1
